import { NgModule, CUSTOM_ELEMENTS_SCHEMA, APP_INITIALIZER } from '@angular/core';
import { RouterModule, Routes } from '@angular/router';
import { HttpClientModule } from '@angular/common/http';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { BrowserModule } from '@angular/platform-browser';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { LocationStrategy, HashLocationStrategy } from '@angular/common';

// Load the previously loaded theme in app startup.
// This is referenced from the APP_INITIALIZER in the module providers.
import { ruiLoadTheme } from '@jha/rui-wc/js/rui-theming';
export function restoreThemeOnAppStartup() {
	return () => {
        if (typeof (Storage) !== "undefined") {
            let themeFile = localStorage.getItem("rui-theme");
            if (themeFile) {
                ruiLoadTheme(themeFile, './assets/rui-themes');
            }
        }
    };
}

// Responsive UI web components
import '@jha/rui-wc/components/rui-buttons/rui-buttons-imports';
import '@jha/rui-wc/components/rui-data-pager/rui-data-pager-imports';
import '@jha/rui-wc/components/rui-dialog-box/rui-dialog-box-imports';
import '@jha/rui-wc/components/rui-header/rui-header-imports';
import '@jha/rui-wc/components/rui-icon/rui-icon-imports';
import '@jha/rui-wc/components/rui-input/rui-input-imports';
import '@jha/rui-wc/components/rui-layout/rui-layout-imports';
import '@jha/rui-wc/components/rui-mega-menu/rui-mega-menu-imports';
import '@jha/rui-wc/components/rui-master-detail/rui-master-detail-imports';
import '@jha/rui-wc/components/rui-nav/rui-nav-imports';
import '@jha/rui-wc/components/rui-notifications/rui-notifications-imports';
import '@jha/rui-wc/components/rui-password-strength-meter/rui-password-strength-meter-imports';
import '@jha/rui-wc/components/rui-percentage-circle/rui-percentage-circle-imports';
import '@jha/rui-wc/components/rui-progress-bar/rui-progress-bar-imports';
import '@jha/rui-wc/components/rui-process-timeline/rui-process-timeline-imports';
import '@jha/rui-wc/components/rui-rating/rui-rating-imports';
import '@jha/rui-wc/components/rui-record-detail/rui-record-detail-imports';
import '@jha/rui-wc/components/rui-session-timeout/rui-session-timeout-imports';
import '@jha/rui-wc/components/rui-tabs/rui-tabs-imports';
import '@jha/rui-wc/components/rui-tile/rui-tile-imports';
import '@jha/rui-wc/components/rui-tooltip/rui-tooltip-imports';

// Responsive UI Angular elements
import { JhaFormsModule } from '@jha/rui-angular/jha-forms';
import { JhaPipesModule } from '@jha/rui-angular/jha-pipes';
import { JhaResponsiveCoreModule } from '@jha/rui-angular/jha-responsive-core';
import { JhaLayoutModule } from '@jha/rui-angular/jha-layout';
import { JhaUnsavedChangesModule } from '@jha/rui-angular/jha-unsaved-changes';
import { JhaUnsavedChangesGuard } from '@jha/rui-angular/jha-unsaved-changes';

// 3rd party imports
import { TypeaheadModule } from 'ngx-bootstrap/typeahead';
import { CalendarModule } from 'primeng/calendar';
import { CheckboxModule } from 'primeng/checkbox';
import { ContextMenuModule } from 'primeng/contextmenu';
import { DropdownModule } from 'primeng/dropdown';
import { FileUploadModule } from 'primeng/fileupload';
import { InputMaskModule } from 'primeng/inputmask';
import { InputNumberModule } from 'primeng/inputnumber';
import { MultiSelectModule } from 'primeng/multiselect';
import { PickListModule } from 'primeng/picklist';
import { RadioButtonModule } from 'primeng/radiobutton';
import { StepsModule } from 'primeng/steps';
import { TableModule } from 'primeng/table';
import { TreeModule } from 'primeng/tree';
import { TreeTableModule } from 'primeng/treetable';

import { MatLegacyChipsModule } from '@angular/material/legacy-chips';
import { MatLegacySelectModule } from '@angular/material/legacy-select';
import { MatLegacyTabsModule } from '@angular/material/legacy-tabs';

import { AgGridModule } from 'ag-grid-angular';
import { AngularSplitModule } from 'angular-split';

// Product Services
import { LogInOutService } from './services/log-in-out/log-in-out.service';
import { DataService } from './services/data/data.service';

// Guards
import { LoggedInGuard } from './services/log-in-out/logged-in.guard';

// Components
import { AppComponent } from './app.component';
import { MainHeaderComponent } from './components/main-header/main-header.component';
import { MainNavComponent } from './components/main-nav/main-nav.component';
import { HomeComponent } from './components/home/home.component';
import { AboutComponent } from './components/about/about.component';

// Pattern views
import { AnimationComponent } from './components/responsive-patterns/animation/animation.component';
import { BreadcrumbsComponent } from './components/responsive-patterns/breadcrumbs/breadcrumbs.component';
import { BusyIndicatorComponent } from './components/responsive-patterns/busy-indicator/busy-indicator.component';
import { ButtonsComponent } from './components/responsive-patterns/buttons/buttons.component';
import { ChangePasswordComponent } from './components/responsive-patterns/change-password/change-password.component';
import { ChartsComponent } from './components/responsive-patterns/charts/charts.component';
import { ContentLayoutComponent } from './components/responsive-patterns/content-layout/content-layout.component';
import { ContextMenuComponent } from './components/responsive-patterns/context-menu/context-menu.component';
import { CrudComponent } from './components/responsive-patterns/crud/crud.component';
import { DashboardComponent } from './components/responsive-patterns/dashboard/dashboard.component';
import { DataDenseComponent } from './components/responsive-patterns/data-dense/data-dense.component';
import { DialogBoxComponent } from './components/responsive-patterns/dialog-boxes/dialog-boxes.component';
import { DisplayBlockComponent } from './components/responsive-patterns/display-block/display-block.component';
import { DisplayBlockCollapseComponent } from './components/responsive-patterns/display-block-collapse/display-block-collapse.component';
import { DragAndDropComponent } from './components/responsive-patterns/drag-and-drop/drag-and-drop.component';
import { EditingFormDataComponent } from './components/responsive-patterns/editing-form-data/editing-form-data.component';
import { EnterpriseIconsComponent } from './components/responsive-patterns/enterprise-icons/enterprise-icons.component';
import { FileUploadComponent } from './components/responsive-patterns/file-upload/file-upload.component';
import { FixedLayoutComponent } from './components/responsive-patterns/fixed-layout/fixed-layout.component';
import { FixedLayoutSplitterComponent } from './components/responsive-patterns/fixed-layout-splitter/fixed-layout-splitter.component';
import { GridComponent } from './components/responsive-patterns/grid/grid.component';
import { GridComplexComponent } from './components/responsive-patterns/grid-complex/grid-complex.component';
import { GridTreeComponent } from './components/responsive-patterns/grid-tree/grid-tree.component';
import { GroupBoxesComponent } from './components/responsive-patterns/group-boxes/group-boxes.component';
import { HeadshotPhotosComponent } from './components/responsive-patterns/headshot-photos/headshot-photos.component';
import { InactivityTimeoutComponent } from './components/responsive-patterns/inactivity-timeout/inactivity-timeout.component';
import { LargeDatasetsComponent } from './components/responsive-patterns/large-datasets/large-datasets.component';
import { LargeFormWizardComponent } from './components/responsive-patterns/large-form-wizard/large-form-wizard.component';
import { LargeFormAccordionComponent } from './components/responsive-patterns/large-form-accordion/large-form-accordion.component';
import { ListCategorizationComponent } from './components/responsive-patterns/list-categorization/list-categorization.component';
import { LogInComponent } from './components/responsive-patterns/log-in/log-in.component';
import { MasterDetailButtonsComponent } from './components/responsive-patterns/master-detail-buttons/master-detail-buttons.component';
import { MasterDetailGridComponent } from './components/responsive-patterns/master-detail-grid/master-detail-grid.component';
import { MasterDetailGridMultipleComponent } from './components/responsive-patterns/master-detail-grid-multiple/master-detail-grid-multiple.component';
import { MasterDetailTreeViewComponent } from './components/responsive-patterns/master-detail-tree-view/master-detail-tree-view.component';
import { NotificationsComponent } from './components/responsive-patterns/notifications/notifications.component';
import { PercentageCircleComponent } from './components/responsive-patterns/percentage-circle/percentage-circle.component';
import { PlugInComponent } from './components/responsive-patterns/plug-in/plug-in.component';
import { ProcessTimelineComponent } from './components/responsive-patterns/process-timeline/process-timeline.component';
import { ProgressBarComponent } from './components/responsive-patterns/progress-bar/progress-bar.component';
import { RatingComponent } from './components/responsive-patterns/rating/rating.component';
import { RecordDetailComponent } from './components/responsive-patterns/record-detail/record-detail.component';
import { ResponsiveLayoutComponent } from './components/responsive-patterns/responsive-layout/responsive-layout.component';
import { ResponsiveTableComponent } from './components/responsive-patterns/responsive-table/responsive-table.component';
import { SearchComponent } from './components/responsive-patterns/search/search.component';
import { SessionTimeoutComponent } from './components/responsive-patterns/session-timeout/session-timeout.component';
import { SliderComponent } from './components/responsive-patterns/slider/slider.component';
import { StackedViewsComponent } from './components/responsive-patterns/stacked-views/stacked-views.component';
import { SelectAddressComponent } from './components/responsive-patterns/stacked-views/components/select-address/select-address.component';
import { EditAddressComponent } from './components/responsive-patterns/stacked-views/components/edit-address/edit-address.component';
import { StatusIconComponent } from './components/responsive-patterns/status-icon/status-icon.component';
import { StepsComponent } from './components/responsive-patterns/steps/steps.component';
import { SubheaderComponent } from './components/responsive-patterns/subheader/subheader.component';
import { TabsComponent } from './components/responsive-patterns/tabs/tabs.component';
import { TagsComponent } from './components/responsive-patterns/tags/tags.component';
import { TemplatedListComponent } from './components/responsive-patterns/templated-list/templated-list.component';
import { TextComponent } from './components/responsive-patterns/text/text.component';
import { ThemedSvgImagesComponent } from './components/responsive-patterns/themed-svg-images/themed-svg-images.component';
import { ThemingComponent } from './components/responsive-patterns/theming/theming.component';
import { TilesComponent } from './components/responsive-patterns/tiles/tiles.component';
import { ToolbarComponent } from './components/responsive-patterns/toolbar/toolbar.component';
import { TooltipsComponent } from './components/responsive-patterns/tooltips/tooltips.component';
import { TreeViewComponent } from './components/responsive-patterns/tree-view/tree-view.component';
import { UnsavedChangesComponent } from './components/responsive-patterns/unsaved-changes/unsaved-changes.component';
import { WizardComponent } from './components/responsive-patterns/wizard/wizard.component';

// ag-Grid Renderers
import { ButtonRendererComponent } from './components/grid-renderers/button-renderer.component';
import { ButtonGroupRendererComponent } from './components/grid-renderers/button-group-renderer.component';
import { CheckboxRendererComponent } from './components/grid-renderers/checkbox-renderer.component';
import { DeleteButtonRendererComponent } from './components/grid-renderers/delete-button-renderer.component';
import { DropdownRendererComponent } from './components/grid-renderers/dropdown-renderer.component';
import { EditDeleteButtonRendererComponent } from './components/grid-renderers/edit-delete-button-renderer.component';
import { HeaderAddButtonRendererComponent } from './components/grid-renderers/header-add-button-renderer.component';
import { InputRendererComponent } from './components/grid-renderers/input-renderer.component';
import { InputNumberRendererComponent } from './components/grid-renderers/inputnumber-renderer.component';
import { MatselectRendererComponent } from './components/grid-renderers/matselect-renderer.component';
import { MultiselectRendererComponent } from './components/grid-renderers/multiselect-renderer.component';
import { SaveCancelButtonRendererComponent } from './components/grid-renderers/save-cancel-button-renderer.component';
import { SelectRendererComponent } from './components/grid-renderers/select-renderer.component';
import { ToggleRendererComponent } from './components/grid-renderers/toggle-renderer.component';

// Routes
const routes: Routes = [
	{ path: '', redirectTo: '/Home', pathMatch: 'full' },
	{ path: 'Home', component: HomeComponent },
    { path: 'About', component: AboutComponent },

	{ path: 'Animation', component: AnimationComponent },
	{ path: 'Breadcrumbs', component: BreadcrumbsComponent },
	{ path: 'BusyIndicator', component: BusyIndicatorComponent },
	{ path: 'Buttons', component: ButtonsComponent },
	{ path: 'ChangePassword', component: ChangePasswordComponent },
	{ path: 'Charts', component: ChartsComponent },
	{ path: 'Crud', component: CrudComponent, canDeactivate: [JhaUnsavedChangesGuard]  },
	{ path: 'ContentLayout', component: ContentLayoutComponent },
	{ path: 'ContextMenu', component: ContextMenuComponent },
	{ path: 'Dashboard', component: DashboardComponent },
	{ path: 'DataDense', component: DataDenseComponent },
	{ path: 'DialogBox', component: DialogBoxComponent },
	{ path: 'DisplayBlock', component: DisplayBlockComponent },
	{ path: 'DisplayBlockCollapse', component: DisplayBlockCollapseComponent },
	{ path: 'DragAndDrop', component: DragAndDropComponent },
	{ path: 'EditingFormData', component: EditingFormDataComponent, canDeactivate: [JhaUnsavedChangesGuard]  },
	{ path: 'EnterpriseIcons', component: EnterpriseIconsComponent },
	{ path: 'FileUpload', component: FileUploadComponent },
	{ path: 'FixedLayout', component: FixedLayoutComponent },
	{ path: 'FixedLayoutSplitter', component: FixedLayoutSplitterComponent },
	{ path: 'Grid', component: GridComponent },
	{ path: 'GridComplex', component: GridComplexComponent },
	{ path: 'GridTree', component: GridTreeComponent },
	{ path: 'GroupBox', component: GroupBoxesComponent },
	{ path: 'HeadshotPhotos', component: HeadshotPhotosComponent },
	{ path: 'InactivityTimeout', component: InactivityTimeoutComponent },
	{ path: 'LargeDatasets', component: LargeDatasetsComponent },
	{ path: 'LargeFormsWizard', component: LargeFormWizardComponent, canDeactivate: [JhaUnsavedChangesGuard]  },
	{ path: 'LargeFormsAccordion', component: LargeFormAccordionComponent, canDeactivate: [JhaUnsavedChangesGuard]  },
	{ path: 'LazyLoad', loadChildren: () => import('./modules/lazy-load/lazy-load.module').then(m => m.LazyLoadModule) },
	{ path: 'ListCategorization', component: ListCategorizationComponent },
	{ path: 'LogIn', component: LogInComponent },
	{ path: 'MasterDetailButtons', component: MasterDetailButtonsComponent },
	{ path: 'MasterDetailGrid', component: MasterDetailGridComponent },
	{ path: 'MasterDetailGridMultiple', component: MasterDetailGridMultipleComponent },
	{ path: 'MasterDetailTreeView', component: MasterDetailTreeViewComponent },
	{ path: 'Notifications', component: NotificationsComponent },
	{ path: 'PercentageCircle', component: PercentageCircleComponent },
	{ path: 'PlugIn', component: PlugInComponent },
	{ path: 'ProcessTimeline', component: ProcessTimelineComponent },
	{ path: 'ProgressBar', component: ProgressBarComponent },
	{ path: 'Rating', component: RatingComponent },
	{ path: 'RecordDetail', component: RecordDetailComponent },
	{ path: 'ResponsiveLayout', component: ResponsiveLayoutComponent },
	{ path: 'ResponsiveTable', component: ResponsiveTableComponent },
	{ path: 'Search', component: SearchComponent },
	{ path: 'SearchPhone', component: SearchComponent },
	{ path: 'SessionTimeout', component: SessionTimeoutComponent },
	{ path: 'Slider', component: SliderComponent },
	{ path: 'StackedViews', component: StackedViewsComponent, canDeactivate: [JhaUnsavedChangesGuard] },
	{ path: 'StatusIcon', component: StatusIconComponent },
	{ path: 'Steps', component: StepsComponent, canDeactivate: [JhaUnsavedChangesGuard]  },
	{ path: 'Subheader', component: SubheaderComponent },
	{ path: 'Tabs', component: TabsComponent },
	{ path: 'Tags', component: TagsComponent },
	{ path: 'TemplatedLists', component: TemplatedListComponent },
	{ path: 'Text', component: TextComponent },
	{ path: 'ThemedSvgImages', component: ThemedSvgImagesComponent },
	{ path: 'Theming', component: ThemingComponent },
	{ path: 'Tiles', component: TilesComponent },
	{ path: 'Toolbar', component: ToolbarComponent },
	{ path: 'Tooltips', component: TooltipsComponent },
	{ path: 'TreeView', component: TreeViewComponent },
	{ path: 'UnsavedChanges', component: UnsavedChangesComponent, canDeactivate: [JhaUnsavedChangesGuard] },
	{ path: 'Wizard', component: WizardComponent, canDeactivate: [JhaUnsavedChangesGuard]  },
];

@NgModule({
	declarations: [
		AppComponent,
		MainHeaderComponent,
		MainNavComponent,
		HomeComponent,
        AboutComponent,
		AnimationComponent,
		BreadcrumbsComponent,
		BusyIndicatorComponent,
		ButtonsComponent,
		ChangePasswordComponent,
		ChartsComponent,
		ContentLayoutComponent,
		ContextMenuComponent,
		CrudComponent,
		DashboardComponent,
		DataDenseComponent,
		DialogBoxComponent,
		DisplayBlockComponent,
		DisplayBlockCollapseComponent,
		DragAndDropComponent,
		EditingFormDataComponent,
		EnterpriseIconsComponent,
		FileUploadComponent,
		FixedLayoutComponent,
		FixedLayoutSplitterComponent,
		GridComponent,
		GridComplexComponent,
		GridTreeComponent,
		GroupBoxesComponent,
		HeadshotPhotosComponent,
		InactivityTimeoutComponent,
		LargeDatasetsComponent,
		LargeFormWizardComponent,
		LargeFormAccordionComponent,
		ListCategorizationComponent,
		LogInComponent,
		MasterDetailButtonsComponent,
		MasterDetailGridComponent,
		MasterDetailGridMultipleComponent,
		MasterDetailTreeViewComponent,
		NotificationsComponent,
		PercentageCircleComponent,
		PlugInComponent,
		ProcessTimelineComponent,
		ProgressBarComponent,
		RatingComponent,
		RecordDetailComponent,
		ResponsiveLayoutComponent,
		ResponsiveTableComponent,
		SearchComponent,
		SessionTimeoutComponent,
		SliderComponent,
		StackedViewsComponent,
		SelectAddressComponent,
		EditAddressComponent,
		StatusIconComponent,
		StepsComponent,
		SubheaderComponent,
		TabsComponent,
		TagsComponent,
		TemplatedListComponent,
		TextComponent,
		ThemedSvgImagesComponent,
		ThemingComponent,
		TilesComponent,
		ToolbarComponent,
		TooltipsComponent,
		TreeViewComponent,
		UnsavedChangesComponent,
		WizardComponent,
		ButtonRendererComponent,
		ButtonGroupRendererComponent,
		CheckboxRendererComponent,
		DeleteButtonRendererComponent,
		DropdownRendererComponent,
		EditDeleteButtonRendererComponent,
		HeaderAddButtonRendererComponent,
		InputRendererComponent,
		InputNumberRendererComponent,
		MatselectRendererComponent,
		MultiselectRendererComponent,
		SaveCancelButtonRendererComponent,
		SelectRendererComponent,
		ToggleRendererComponent
	],
	imports: [
		BrowserModule,
		BrowserAnimationsModule,
		RouterModule.forRoot(routes),
		HttpClientModule,
		FormsModule,
		ReactiveFormsModule,
		JhaFormsModule.forRoot(),
		JhaPipesModule,
		JhaResponsiveCoreModule.forRoot(),
		JhaLayoutModule,
		JhaUnsavedChangesModule.forRoot(),
		TypeaheadModule.forRoot(),
		CalendarModule,
		CheckboxModule,
		ContextMenuModule,
		DropdownModule,
		FileUploadModule,
		InputMaskModule,
		InputNumberModule,
		MultiSelectModule,
		PickListModule,
		RadioButtonModule,
		StepsModule,
		TableModule,
		TreeModule,
		TreeTableModule,
		MatLegacyChipsModule,
		MatLegacySelectModule,
		MatLegacyTabsModule,
		AgGridModule,
		AngularSplitModule
	],
	providers: [
		{ provide: LocationStrategy, useClass: HashLocationStrategy },
        { provide: APP_INITIALIZER, useFactory: restoreThemeOnAppStartup, multi: true },
		LogInOutService,
		LoggedInGuard,
		DataService
	],
	schemas: [
		CUSTOM_ELEMENTS_SCHEMA
	],
	bootstrap: [AppComponent]
})
export class AppModule { }
