import { Component, OnInit, HostListener } from '@angular/core';
import { Router } from '@angular/router';
import { RUI_VERSION } from '@jha/rui-wc/js/rui-constants';

@Component({
    selector: 'main-header',
    templateUrl: './main-header.component.html',
    styleUrls: ['./main-header.component.scss']
})
export class MainHeaderComponent implements OnInit {
    public hideHeader: boolean = false;
    public displayBackButton: boolean = false;
    public displayHeaderSearch: boolean = true;
    public version: string = RUI_VERSION;
    private resizeTimeout: any;

    constructor(private router: Router) { }

    ngOnInit() {
        this.checkWidth();
    }

    // Start/stop vertical scrolling mode if using fixed layout
    @HostListener('window:resize', ['$event'])
    onResize(event: Event) {
        // Using a timer so we don't run this processing on every single fire of this event
        if (this.resizeTimeout) {
            clearTimeout(this.resizeTimeout);
        }
        this.resizeTimeout = setTimeout((() => {
            this.checkWidth();
        }).bind(this), 250);
    }

    public checkWidth() {
        this.displayHeaderSearch = window.innerWidth > 1024;
    }

    backButtonClicked(): void {
        window.history.back();
    }

    public search(e: CustomEvent): void {
        this.router.navigate(['Search', { search: e.detail }]);
    }
}
