import { Component, OnInit } from '@angular/core';
import { UntypedFormGroup, UntypedFormControl, Validators } from '@angular/forms';
import { PasswordInvalidValidator } from './password-invalid-validator.directive';
import { PasswordMismatchValidator } from './password-mismatch-validator.directive';
import { PasswordNoLowercaseValidator } from './password-no-lowercase-validator.directive';
import { PasswordNoSymbolValidator } from './password-no-symbol-validator.directive';
import { PasswordNoUppercaseValidator } from './password-no-uppercase-validator.directive';
import { PasswordTooShortValidator } from './password-too-short-validator.directive';

@Component({
	selector: 'change-password',
	templateUrl: './change-password.component.html',
	styleUrls: ['./change-password.component.scss']
})
export class ChangePasswordComponent implements OnInit {
	public passwordChanged: boolean = false;
	public showPasswords: boolean = false;
	public currentPasswordVal: string = '';
	public newPasswordVal: string = '';
	public newPasswordRepeatedVal: string = '';
	public passwordStrengthValue: number = 100;
	public passwordStrengthDescription: string = 'Weak';
	public showSymbolCharacters: boolean = false;

	public changePasswordForm: UntypedFormGroup = new UntypedFormGroup({
		'currentPassword': new UntypedFormControl(
			this.currentPasswordVal,
			{
				validators: [Validators.required, PasswordInvalidValidator],
				updateOn: 'blur'
			}
		),
		'newPassword': new UntypedFormControl(
			this.newPasswordVal,
			{
				validators: [Validators.required, PasswordTooShortValidator, PasswordNoUppercaseValidator, PasswordNoLowercaseValidator, PasswordNoSymbolValidator],
				updateOn: 'blur'
			}
		),
		'newPasswordRepeated': new UntypedFormControl(
			this.newPasswordRepeatedVal,
			{
				validators: Validators.required,
				updateOn: 'blur'
			}
		)
	},
		{
			// Record-level validations
			validators: PasswordMismatchValidator,
			updateOn: 'blur'
		}
	);

	get currentPassword(): any { return this.changePasswordForm.get('currentPassword'); }
	get newPassword(): any { return this.changePasswordForm.get('newPassword'); }
	get newPasswordRepeated(): any { return this.changePasswordForm.get('newPasswordRepeated'); }


	ngOnInit(): void {
		this.setPasswordStrength();
	}

	public changePassword(): void {
		if (this.changePasswordForm.valid) {

			// PASSWORD CHANGE LOGIC GOES HERE

			this.passwordChanged = true;
		}
	}

	public setPasswordStrength(): void {
		this.passwordStrengthValue = 100;

		// Determine password strength
		if (this.newPassword.errors) {
			if (this.newPassword.errors.passwordTooShort) {
				this.passwordStrengthValue = this.passwordStrengthValue - 25;
			}
			if (this.newPassword.errors.passwordNoUppercase) {
				this.passwordStrengthValue = this.passwordStrengthValue - 25;
			}
			if (this.newPassword.errors.passwordNoLowercase) {
				this.passwordStrengthValue = this.passwordStrengthValue - 25;
			}
			if (this.newPassword.errors.passwordNoSymbol) {
				this.passwordStrengthValue = this.passwordStrengthValue - 25;
			}
		}

		// Update password strength description based on latest value
		if (this.passwordStrengthValue > 0 && this.passwordStrengthValue <= 33) {
			this.passwordStrengthDescription = 'Weak';
		} else if (this.passwordStrengthValue >= 34 && this.passwordStrengthValue <= 67) {
			this.passwordStrengthDescription = 'Medium';
		} else if (this.passwordStrengthValue >= 67) {
			this.passwordStrengthDescription = 'Strong';
		}
	}

	public openPattern(url: string): void {
		const win = window.open(url, '_blank');
		win.focus();
	}
}
