import { Component, OnInit, ElementRef, HostListener } from '@angular/core';
import Chart from 'chart.js/auto';

@Component({
	// tslint:disable-next-line:component-selector
	selector: 'charts',
	templateUrl: './charts.component.html',
	styleUrls: ['./charts.component.scss']
})
export class ChartsComponent implements OnInit {
	private subscription: any = null;
    private barChart = null;
    private doughnutChart = null;
    private pieChart = null;
    private lineChart = null;
    private bubbleChart = null;
    private areaChart = null;
    private animateCharts: Boolean = true;
    private chartFontColor = '';
    private chartGridColor = '';
    private chartDataSeriesColor1 = '';
    private chartDataSeriesTranslucentColor1 = '';
    private chartDataSeriesColor2 = '';
    private chartDataSeriesTranslucentColor2 = '';
    private chartDataSeriesColor3 = '';
    private chartDataSeriesTranslucentColor3 = '';
    private chartDataSeriesColor4 = '';
    private chartDataSeriesTranslucentColor4 = '';
    private chartDataSeriesColor5 = '';
    private chartDataSeriesTranslucentColor5 = '';
    private chartDataSeriesColor6 = '';
    private chartDataSeriesTranslucentColor6 = '';
    private chartDataSeriesColor7 = '';
    private chartDataSeriesTranslucentColor7 = '';
    private chartDataSeriesColor8 = '';
    private chartDataSeriesTranslucentColor8 = '';
    private chartDataSeriesColor9 = '';
    private chartDataSeriesTranslucentColor9 = '';
    private chartDataSeriesColor10 = '';
    private chartDataSeriesTranslucentColor10 = '';

	constructor(private elRef:ElementRef) { }

	ngOnInit(): void {
        // Check if the user requests no animation
        const mediaQuery = window.matchMedia("(prefers-reduced-motion: reduce)");
        if (!mediaQuery || mediaQuery.matches) {
            this.animateCharts = false;
        }

        // Set up charts in page
        this.setUpCharts();
	}

    @HostListener('window:rui-load-theme', ['$event'])
    private themeChanged(e) {
        this.setUpCharts();
    }

    // Set up all charts in page
    setUpCharts() {
        setTimeout(() => {
            // Grab chart colors from themed CSS vars
            this.getChartColors();

            // Set up each chart
            this.setUpBarChart();
            this.setUpDoughnutChart();
            this.setUpPieChart();
            this.setUpLineChart();
            this.setUpBubbleChart();
            this.setUpAreaChart();
        }, 100);
    }

    // Grab chart colors from themed CSS vars
    getChartColors() {
        const styles = getComputedStyle(this.elRef.nativeElement);
        this.chartFontColor = String(styles.getPropertyValue('--text-regular'));
        this.chartGridColor = String(styles.getPropertyValue('--chart-grid-color'));
        this.chartDataSeriesColor1 = String(styles.getPropertyValue('--chart-data-series-color-1'));
        this.chartDataSeriesTranslucentColor1 = String(styles.getPropertyValue('--chart-data-series-translucent-color-1'));
        this.chartDataSeriesColor2 = String(styles.getPropertyValue('--chart-data-series-color-2'));
        this.chartDataSeriesTranslucentColor2 = String(styles.getPropertyValue('--chart-data-series-translucent-color-2'));
        this.chartDataSeriesColor3 = String(styles.getPropertyValue('--chart-data-series-color-3'));
        this.chartDataSeriesTranslucentColor3 = String(styles.getPropertyValue('--chart-data-series-translucent-color-3'));
        this.chartDataSeriesColor4 = String(styles.getPropertyValue('--chart-data-series-color-4'));
        this.chartDataSeriesTranslucentColor4 = String(styles.getPropertyValue('--chart-data-series-translucent-color-4'));
        this.chartDataSeriesColor5 = String(styles.getPropertyValue('--chart-data-series-color-5'));
        this.chartDataSeriesTranslucentColor5 = String(styles.getPropertyValue('--chart-data-series-translucent-color-5'));
        this.chartDataSeriesColor6 = String(styles.getPropertyValue('--chart-data-series-color-6'));
        this.chartDataSeriesTranslucentColor6 = String(styles.getPropertyValue('--chart-data-series-translucent-color-6'));
        this.chartDataSeriesColor7 = String(styles.getPropertyValue('--chart-data-series-color-7'));
        this.chartDataSeriesTranslucentColor7 = String(styles.getPropertyValue('--chart-data-series-translucent-color-7'));
        this.chartDataSeriesColor8 = String(styles.getPropertyValue('--chart-data-series-color-8'));
        this.chartDataSeriesTranslucentColor8 = String(styles.getPropertyValue('--chart-data-series-translucent-color-8'));
        this.chartDataSeriesColor9 = String(styles.getPropertyValue('--chart-data-series-color-9'));
        this.chartDataSeriesTranslucentColor9 = String(styles.getPropertyValue('--chart-data-series-translucent-color-9'));
        this.chartDataSeriesColor10 = String(styles.getPropertyValue('--chart-data-series-color-10'));
        this.chartDataSeriesTranslucentColor10 = String(styles.getPropertyValue('--chart-data-series-translucent-color-10'));
    }

    setUpBarChart() {
        const canvas = document.querySelector('#barChart');
        if (canvas) {
            if (this.barChart) {
                // If chart already exists, update it with current theme's chart colors
                this.barChart.data.datasets[0].backgroundColor = this.chartDataSeriesTranslucentColor1;
                this.barChart.data.datasets[1].backgroundColor = this.chartDataSeriesTranslucentColor2;
                this.barChart.options.plugins.legend.labels.color = this.chartFontColor;
                this.barChart.options.scales.x.ticks.color = this.chartFontColor;
                this.barChart.options.scales.y.ticks.color = this.chartFontColor;
                this.barChart.options.scales.x.grid.color = this.chartGridColor;
                this.barChart.options.scales.y.grid.color = this.chartGridColor;
                this.barChart.update();
            } else {
                // Otherwise create a new chart using current theme's chart colors
                this.barChart = new Chart(canvas as any, {
                    type: 'bar',
                    data: {
                        labels: ['January', 'February', 'March', 'April', 'May', 'June', 'July'],
                        datasets: [
                            {
                                label: 'Domestic',
                                backgroundColor: this.chartDataSeriesTranslucentColor1,
                                data: [65, 59, 80, 81, 56, 55, 40]
                            },
                            {
                                label: 'International',
                                backgroundColor: this.chartDataSeriesTranslucentColor2,
                                data: [28, 48, 40, 19, 86, 27, 90]
                            }
                        ]
                    },
                    options: {
                        ...(!this.animateCharts && { animation: false }),
                        responsive: true,
                        maintainAspectRatio: false,
                        plugins: {
                            legend: {
                                labels: {
                                    color: this.chartFontColor
                                }
                            }
                        },
                        scales:
                        {
                            x: {
                                grid: { color: this.chartGridColor },
                                ticks: { color: this.chartFontColor }
                            },
                            y: {
                                grid: { color: this.chartGridColor },
                                ticks: { color: this.chartFontColor }
                            }
                        }
                    }
                });
            }
        }
    }

    setUpDoughnutChart() {
        const canvas = document.querySelector('#doughnutChart');
        if (canvas) {
            if (this.doughnutChart) {
                // If chart already exists, update it with current theme's chart colors
                this.doughnutChart.data.datasets[0].backgroundColor = [
                    this.chartDataSeriesColor1,
                    this.chartDataSeriesColor2,
                    this.chartDataSeriesColor3,
                    this.chartDataSeriesColor4
                ];
                this.doughnutChart.options.plugins.legend.labels.color = this.chartFontColor;
                this.doughnutChart.update();
            } else {
                // Otherwise create a new chart using current theme's chart colors
                this.doughnutChart = new Chart(canvas as any, {
                    type: 'doughnut',
                    data: {
                        labels: ['US', 'Europe', 'Asia', 'Africa'],
                        datasets: [
                            {
                                backgroundColor: [
                                    this.chartDataSeriesColor1,
                                    this.chartDataSeriesColor2,
                                    this.chartDataSeriesColor3,
                                    this.chartDataSeriesColor4
                                ],
                                borderWidth: 0,
                                data: [65, 59, 80, 81]
                            }
                        ]
                    },
                    options: {
                        ...(!this.animateCharts && { animation: false }),
                        responsive: true,
                        maintainAspectRatio: false,
                        plugins: {
                            legend: {
                                labels: {
                                    color: this.chartFontColor
                                }
                            }
                        }
                    } as any
                });
            }
        }
    }

    setUpPieChart() {
        const canvas = document.querySelector('#pieChart');
        if (canvas) {
            if (this.pieChart) {
                // If chart already exists, update it with current theme's chart colors
                this.pieChart.data.datasets[0].backgroundColor = [
                    this.chartDataSeriesColor4,
                    this.chartDataSeriesColor5,
                    this.chartDataSeriesColor6,
                    this.chartDataSeriesColor7,
                    this.chartDataSeriesColor8
                ];
                this.pieChart.options.plugins.legend.labels.color = this.chartFontColor;
                this.pieChart.update();
            } else {
                // Otherwise create a new chart using current theme's chart colors
                this.pieChart = new Chart(canvas as any, {
                    type: 'pie',
                    data: {
                        labels: ['Home', 'Auto & Transport', 'Food & Dining', 'Bills & Utilities', 'Travel'],
                        datasets: [
                            {
                                backgroundColor: [
                                    this.chartDataSeriesColor4,
                                    this.chartDataSeriesColor5,
                                    this.chartDataSeriesColor6,
                                    this.chartDataSeriesColor7,
                                    this.chartDataSeriesColor8
                                ],
                                borderWidth: 0,
                                data: [29, 13, 26, 18, 14]
                            }
                        ]
                    },
                    options: {
                        ...(!this.animateCharts && { animation: false }),
                        responsive: true,
                        maintainAspectRatio: false,
                        plugins: {
                            legend: {
                                labels: {
                                    color: this.chartFontColor
                                }
                            }
                        }
                    } as any
                });
            }
        }
    }

    setUpLineChart() {
        const canvas = document.querySelector('#lineChart');
        if (canvas) {
            if (this.lineChart) {
                // If chart already exists, update it with current theme's chart colors
                this.lineChart.data.datasets[0].backgroundColor = this.chartDataSeriesColor5;
                this.lineChart.data.datasets[0].borderColor = this.chartDataSeriesColor5;
                this.lineChart.data.datasets[1].backgroundColor = this.chartDataSeriesColor10;
                this.lineChart.data.datasets[1].borderColor = this.chartDataSeriesColor10;
                this.lineChart.options.plugins.legend.labels.color = this.chartFontColor;
                this.lineChart.options.scales.x.ticks.color = this.chartFontColor;
                this.lineChart.options.scales.y.ticks.color = this.chartFontColor;
                this.lineChart.options.scales.x.grid.color = this.chartGridColor;
                this.lineChart.options.scales.y.grid.color = this.chartGridColor;
                this.lineChart.update();
            } else {
                // Otherwise create a new chart using current theme's chart colors
                this.lineChart = new Chart(canvas as any, {
                    type: 'line',
                    data: {
                        labels: ['January', 'February', 'March', 'April', 'May', 'June', 'July'],
                        datasets: [
                            {
                                label: 'Online',
                                backgroundColor: this.chartDataSeriesColor5,
                                borderColor: this.chartDataSeriesColor5,
                                data: [65, 59, 80, 81, 56, 55, 40]
                            },
                            {
                                label: 'Branch',
                                backgroundColor: this.chartDataSeriesColor10,
                                borderColor: this.chartDataSeriesColor10,
                                data: [28, 48, 40, 19, 86, 27, 25]
                            }
                        ]
                    },
                    options: {
                        ...(!this.animateCharts && { animation: false }),
                        responsive: true,
                        maintainAspectRatio: false,
                        plugins: {
                            legend: {
                                labels: {
                                    color: this.chartFontColor
                                }
                            }
                        },
                        scales:
                        {
                            x: {
                                grid: { color: this.chartGridColor },
                                ticks: { color: this.chartFontColor }
                            },
                            y: {
                                grid: { color: this.chartGridColor },
                                ticks: { color: this.chartFontColor }
                            }
                        }
                    }
                });
            }
        }
    }

    setUpBubbleChart() {
        const canvas = document.querySelector('#bubbleChart');
        if (canvas) {
            if (this.bubbleChart) {
                // If chart already exists, update it with current theme's chart colors
                this.bubbleChart.data.datasets[0].backgroundColor = this.chartDataSeriesTranslucentColor8;
                this.bubbleChart.data.datasets[0].borderColor = this.chartDataSeriesColor8;
                this.bubbleChart.data.datasets[1].backgroundColor = this.chartDataSeriesTranslucentColor3;
                this.bubbleChart.data.datasets[1].borderColor = this.chartDataSeriesColor3;
                this.bubbleChart.options.plugins.legend.labels.color = this.chartFontColor;
                this.bubbleChart.options.scales.x.ticks.color = this.chartFontColor;
                this.bubbleChart.options.scales.y.ticks.color = this.chartFontColor;
                this.bubbleChart.options.scales.x.grid.color = this.chartGridColor;
                this.bubbleChart.options.scales.y.grid.color = this.chartGridColor;
                this.bubbleChart.update();
            } else {
                // Otherwise create a new chart using current theme's chart colors
                this.bubbleChart = new Chart(canvas as any, {
                    type: 'bubble',
                    data: {
                        labels: ['January', 'February', 'March', 'April', 'May', 'June', 'July'],
                        datasets: [
                            {
                                label: 'Production',
                                backgroundColor: this.chartDataSeriesTranslucentColor8,
                                borderColor: this.chartDataSeriesColor8,
                                borderWidth: 0.5,
                                data: [
                                    {
                                        x: 63.710562414266114,
                                        y: 93.07313100137175,
                                        r: 14.43235596707819
                                    },
                                    {
                                        x: 24.56061385459534,
                                        y: 59.40157750342936,
                                        r: 6.520447530864198
                                    },
                                    {
                                        x: 37.95696159122085,
                                        y: 58.83187585733882,
                                        r: 6.640946502057613
                                    },
                                    {
                                        x: 45.56627229080933,
                                        y: 33.030692729766805,
                                        r: 8.960519547325102
                                    },
                                    {
                                        x: 89.05521262002743,
                                        y: 23.664694787379972,
                                        r: 7.645833333333334
                                    },
                                    {
                                        x: 10.090449245541839,
                                        y: 72.400548696845,
                                        r: 6.863554526748971
                                    },
                                    {
                                        x: 50.3386488340192,
                                        y: 20.904921124828533,
                                        r: 10.780349794238683
                                    }
                                ],
                            },
                            {
                                label: 'Test',
                                backgroundColor: this.chartDataSeriesTranslucentColor3,
                                borderColor: this.chartDataSeriesColor3,
                                borderWidth: 0.5,
                                data: [
                                    {
                                        x: 92.2153635116598,
                                        y: 16.228137860082303,
                                        r: 10.904235253772292
                                    },
                                    {
                                        x: 74.05306927297669,
                                        y: 88.72942386831276,
                                        r: 14.350351508916324
                                    },
                                    {
                                        x: 97.32596021947873,
                                        y: 49.88811728395062,
                                        r: 8.051097393689986
                                    },
                                    {
                                        x: 3.7007030178326477,
                                        y: 41.37088477366255,
                                        r: 6.173139574759945
                                    },
                                    {
                                        x: 34.84396433470508,
                                        y: 4.8443930041152266,
                                        r: 12.883144718792867
                                    },
                                    {
                                        x: 32.42241083676269,
                                        y: 81.9753086419753,
                                        r: 12.34777949245542
                                    },
                                    {
                                        x: 38.10270919067215,
                                        y: 14.430298353909466,
                                        r: 8.733710562414267
                                    }
                                ]
                            }
                        ]
                    },
                    options: {
                        ...(!this.animateCharts && { animation: false }),
                        responsive: true,
                        maintainAspectRatio: false,
                        plugins: {
                            legend: {
                                labels: {
                                    color: this.chartFontColor
                                }
                            }
                        },
                        scales:
                        {
                            x: {
                                grid: { color: this.chartGridColor },
                                ticks: { color: this.chartFontColor }
                            },
                            y: {
                                grid: { color: this.chartGridColor },
                                ticks: { color: this.chartFontColor }
                            }
                        }
                    }
                });
            }
        }
    }

    setUpAreaChart() {
        const canvas = document.querySelector('#areaChart');
        if (canvas) {
            if (this.areaChart) {
                // If chart already exists, update it with current theme's chart colors
                this.areaChart.data.datasets[0].backgroundColor = this.chartDataSeriesTranslucentColor1;
                this.areaChart.data.datasets[0].borderColor = this.chartDataSeriesColor1;
                this.areaChart.options.plugins.legend.labels.color = this.chartFontColor;
                this.areaChart.options.scales.x.ticks.color = this.chartFontColor;
                this.areaChart.options.scales.y.ticks.color = this.chartFontColor;
                this.areaChart.options.scales.x.grid.color = this.chartGridColor;
                this.areaChart.options.scales.y.grid.color = this.chartGridColor;
                this.areaChart.update();
            } else {
                // Otherwise create a new chart using current theme's chart colors
                this.areaChart = new Chart(canvas as any, {
                    type: 'line',
                    data: {
                        labels: ['January', 'February', 'March', 'April', 'May', 'June', 'July'],
                        datasets: [
                            {
                                label: 'Deposit Balances',
                                backgroundColor: this.chartDataSeriesTranslucentColor1,
                                borderColor: this.chartDataSeriesColor1,
                                borderWidth: 0.5,
                                data: [65, 59, 80, 81, 56, 55, 40],
                                fill: 'start'
                            }]
                    },
                    options: {
                        ...(!this.animateCharts && { animation: false }),
                        responsive: true,
                        maintainAspectRatio: false,
                        plugins: {
                            legend: {
                                labels: {
                                    color: this.chartFontColor
                                }
                            }
                        },
                        scales:
                        {
                            x: {
                                grid: { color: this.chartGridColor },
                                ticks: { color: this.chartFontColor }
                            },
                            y: {
                                grid: { color: this.chartGridColor },
                                ticks: { color: this.chartFontColor }
                            }
                        }
                    }
                });
            }
        }
    }

	public openPattern(url: string): void {
		const win = window.open(url, '_blank');
		win.focus();
	}
}
