import { Component, OnInit, QueryList, ViewChildren } from '@angular/core';
import { MenuItem } from 'primeng/api';
import { ContextMenu } from 'primeng/contextmenu';

@Component({
	selector: 'app-context-menu',
	templateUrl: './context-menu.component.html',
	styleUrls: ['./context-menu.component.scss']
})
export class ContextMenuComponent implements OnInit {
	public displayBlockMenuItems: MenuItem[];
	public templateMenuItems: MenuItem[];
	public selectedAccountNumber: string = '';
	public selectedAccountType: string = '';
	public accountList: any[] =
	[
		{ 'iconType': 'account', 'name': '00214', 'type': 'Demand Deposit', 'balance': 1234.56 },
		{ 'iconType': 'check', 'name': '00017', 'type': 'Checking', 'balance': 415.92 },
		{ 'iconType': 'savings', 'name': '00190', 'type': 'Savings', 'balance': 7624.18 }
	];
	@ViewChildren(ContextMenu) contextMenus: QueryList<ContextMenu>;

	ngOnInit(): void {
		this.displayBlockMenuItems = [
			{
				label: 'Link Item',
				items: [
					{label: 'RouterLink Item', routerLink: ['/Home']},
					{label: 'URL Item', url: 'http://www.google.com'}
				]
			},
			{
				label: 'Event Item',
				items: [
					{label: 'Click Event Item', command: (event) => {
						alert('Context Menu Item Clicked');
					}}
				]
			},
			{
				separator: true,
			},
			{
				label: 'Menu Item',
				routerLink: ['/Home']
			},
			{
				label: 'Disabled Item',
				disabled: true,
				command: (event) => {
					alert('Context Menu Item Clicked');
				}
			}
		];


		this.templateMenuItems = [
			{
				label: 'Update Account',
				items: [
					{label: 'Save Account', command: (event) => {
						alert('Saved ' + this.selectedAccountType + ' Account #' + this.selectedAccountNumber);
					}},
					{label: 'Delete Account', command: (event) => {
						alert('Deleted ' + this.selectedAccountType + ' Account #' + this.selectedAccountNumber);
					}}
				]
			},
			{
				separator: true,
			},
			{label: 'Router Link Item', routerLink: ['/Home']},
			{label: 'URL Item', url: 'http://www.google.com'}
		];
	}

	public onContextMenuClick(event: Event, accountNumber: string, accountType: string): void {
		this.contextMenuClicked(event);
		this.selectedAccountNumber = accountNumber;
		this.selectedAccountType = accountType;
	}

	public contextMenuClicked(e) {
        let contextMenuEvent = new CustomEvent('contextmenu', {
            detail: {},
            bubbles: true,
            composed: true
        });
        dispatchEvent(contextMenuEvent);
    }

	public openPattern(url: string): void {
		const win = window.open(url, '_blank');
		win.focus();
	}
}
