import { Component, OnInit, AfterViewInit } from '@angular/core';
import { UntypedFormGroup, UntypedFormControl, Validators } from '@angular/forms';
import { SelectItem } from 'primeng/api';
import { MatChipInputEvent } from '@angular/material/chips';
import { COMMA, ENTER } from '@angular/cdk/keycodes';
import { lastValueFrom } from 'rxjs';
import { DataService } from '../../../services/data/data.service';
import { JhaUnsavedChangesService } from '@jha/rui-angular/jha-unsaved-changes';
import { CountryAddressTypeValidator } from './country-address-type-validator.directive';
import { AlertGroup } from '../../../classes/alert-group';
import { CustomerDetails } from '../../../classes/customer-details';
import { JhaCustomValidationMap } from '@jha/rui-angular/jha-forms';

declare var Jodit: any;

@Component({
	selector: 'editing-form-data',
	templateUrl: './editing-form-data.component.html',
	styleUrls: ['./editing-form-data.component.scss']
})
export class EditingFormDataComponent implements OnInit, AfterViewInit {
	public loadingCustomerInfo: boolean = true;
	public postingCustomerInfo: boolean = false;
	public customerInfo: CustomerDetails =
		new CustomerDetails('', false, 0, 0, new Date(), 0, 0, false, [], '', '', null, '', '', '', 0, '', '', 'Domestic', '', '', '', '', null, '', null, null);
	public accountTypeList: SelectItem[] = [];
	public monthlyIncome: number = 0;
	public annualIncome: number = 0;
	public stateList: SelectItem[] = [];
	public countryList: string[] = [];
	public countyList: SelectItem[] = [];
	public alertGroups: AlertGroup[] = [];
	public showDialog: boolean = false;
	public richTextEditor: HTMLTextAreaElement;
	public recentAccounts: SelectItem[] =
	[
		{ label: 'John Doe', value: 'A000007' },
		{ label: 'Kevin Peterson', value: '141' },
		{ label: 'Frank Kristopoulos', value: 'A000013' },
		{ label: 'Fidel Martin', value: '80103' }
	];
	public selectedValues: string[] = ['Anteater', 'Bat', 'Cat', 'Dog'];
	public readonly separatorKeyCodes: number[] = [ENTER, COMMA];
	public accountCategoryList: string[] = ['Customer Account', 'Employee Account', 'Shareholder Account'];
	public animalList: string[] = ['Anteater', 'Bat', 'Cat', 'Dog', 'Elephant', 'Falcon', 'Gorilla', 'Hippo'];
	public typeaheadSelectionInProgess: boolean = false;
	public customValidationMaps: JhaCustomValidationMap[] = [
		new JhaCustomValidationMap('countryAddressTypeMismatch', ['country', 'addressType'])
	];
	public editForm: UntypedFormGroup = new UntypedFormGroup({
		'name': new UntypedFormControl(this.customerInfo.name, Validators.required),
		'ssn': new UntypedFormControl(this.customerInfo.ssn, Validators.required),
		'birthdate': new UntypedFormControl(this.customerInfo.birthdate, Validators.required),
		'lastLogin': new UntypedFormControl(this.customerInfo.lastLogin, Validators.required),
		'grossIncome': new UntypedFormControl(this.customerInfo.grossIncome, Validators.required),
		'interestRate': new UntypedFormControl(this.customerInfo.interestRate, [Validators.required, Validators.min(3), Validators.max(100)]),
		'accountTypes': new UntypedFormControl(this.customerInfo.accountTypes, Validators.required),
		'accountTier': new UntypedFormControl(this.customerInfo.accountTier, Validators.required),
		'accountCategory': new UntypedFormControl(this.customerInfo.accountCategory, Validators.required),
		'recentAccounts': new UntypedFormControl(this.recentAccounts, Validators.required),
		'activeCustomer': new UntypedFormControl(this.customerInfo.activeCustomer),
		'idChecked': new UntypedFormControl(this.customerInfo.idChecked),
		'street': new UntypedFormControl(this.customerInfo.street, Validators.required),
		'city': new UntypedFormControl(this.customerInfo.city, Validators.required),
		'state': new UntypedFormControl(this.customerInfo.state, Validators.required),
		'zip': new UntypedFormControl(this.customerInfo.zip, [Validators.required, Validators.pattern('^[0-9]{5}(?:[0-9]{4})?$')]),
		'county': new UntypedFormControl(this.customerInfo.county, Validators.required),
		'country': new UntypedFormControl(this.customerInfo.country, Validators.required),
		'addressType': new UntypedFormControl(this.customerInfo.addressType),
		'homePhone': new UntypedFormControl(this.customerInfo.homePhone),
		'businessPhone': new UntypedFormControl(this.customerInfo.businessPhone),
		'mobilePhone': new UntypedFormControl(this.customerInfo.mobilePhone),
		'email': new UntypedFormControl(this.customerInfo.email, Validators.email),
		'alerts': new UntypedFormControl(this.customerInfo.alerts, Validators.required),
		'comments': new UntypedFormControl(''),
		'editor': new UntypedFormControl(''),
		'selectedAnimal': new UntypedFormControl('')
	}, { validators: CountryAddressTypeValidator });

	get name(): any { return this.editForm.get('name'); }
	get ssn(): any { return this.editForm.get('ssn'); }
	get birthdate(): any { return this.editForm.get('birthdate'); }
	get lastLogin(): any { return this.editForm.get('lastLogin'); }
	get grossIncome(): any { return this.editForm.get('grossIncome'); }
	get interestRate(): any { return this.editForm.get('interestRate'); }
	get accountTypes(): any { return this.editForm.get('accountTypes'); }
	get accountTier(): any { return this.editForm.get('accountTier'); }
	get accountCategory(): any { return this.editForm.get('accountCategory'); }
	get activeCustomer(): any { return this.editForm.get('activeCustomer'); }
	get idChecked(): any { return this.editForm.get('idChecked'); }
	get street(): any { return this.editForm.get('street'); }
	get city(): any { return this.editForm.get('city'); }
	get state(): any { return this.editForm.get('state'); }
	get zip(): any { return this.editForm.get('zip'); }
	get county(): any { return this.editForm.get('county'); }
	get country(): any { return this.editForm.get('country'); }
	get addressType(): any { return this.editForm.get('addressType'); }
	get homePhone(): any { return this.editForm.get('homePhone'); }
	get businessPhone(): any { return this.editForm.get('businessPhone'); }
	get mobilePhone(): any { return this.editForm.get('mobilePhone'); }
	get email(): any { return this.editForm.get('email'); }
	get alerts(): any { return this.editForm.get('alerts'); }
	get comments(): any { return this.editForm.get('comments'); }
	get editor(): any { return this.editForm.get('editor'); }
	get selectedAnimal(): any { return this.editForm.get('selectedAnimal'); }

	constructor(
		private dataService: DataService,
		private unsavedChangesService: JhaUnsavedChangesService
	) {

		// Capture desktop and mobile dropdowns
		const stateDropDown: any = this.editForm.get('state');

		// Manually subscribe to value changes on state dropdown and propogate to all state form controls
		stateDropDown.valueChanges.subscribe((value: any) => stateDropDown.setValue(value, { emitEvent: false }));

		// Capture multiple account type inputs
		const accountTypeInputs: any = this.editForm.get('accountTypes');

		// Manually subscribe to value changes on account types and propogate to all account type form controls
		accountTypeInputs.valueChanges.subscribe((value: any) => accountTypeInputs.setValue(value, { emitEvent: false }));

		// Capture primeng radio buttons
		const accountCategory: any = this.editForm.get('accountCategory');

		// Manually subscribe to value changes on radio button changes and propogate to all state form controls
		// due to bug in PrimeNG control: https://github.com/primefaces/primeng/issues/9162
		accountCategory.valueChanges.subscribe((value: any) => accountCategory.setValue(value, { emitEvent: false }));

	}

	ngOnInit(): void {
		// Load list of account types
		this.dataService.GetAccountTypeList().then((accountTypeList) => { this.accountTypeList = accountTypeList; });

		// Load list of US states
		this.dataService.GetStateList().then((stateList) => {
			 this.stateList = [];
			 stateList.forEach((stateOption) => { this.stateList.push({ 'label': stateOption.name, 'value': stateOption.value }); });
		});

		// Load list of counties
		this.dataService.GetCountyList().then((countyList) => { this.countyList = countyList; });

		// Load list of countries
		this.dataService.GetCountryList().then((countryList) => { this.countryList = countryList; });

		// Load list of alerts
		this.dataService.getAlertTypeList().then((alertGroups) => { this.alertGroups = alertGroups; });

		// Load customer info
		this.loadingCustomerInfo = true;
		this.dataService.getCustomerDetailInfo().then((customerDetailInfo) => {
			this.loadingCustomerInfo = false;
			this.customerInfo = customerDetailInfo;
			this.name.setValue(customerDetailInfo.name);
			this.ssn.setValue(customerDetailInfo.ssn);
			this.birthdate.setValue(customerDetailInfo.birthdate);
			this.lastLogin.setValue(customerDetailInfo.lastLogin);
			this.grossIncome.setValue(customerDetailInfo.grossIncome);
			this.interestRate.setValue(customerDetailInfo.interestRate);
			this.accountTypes.setValue(customerDetailInfo.accountTypes);
			this.accountTier.setValue(customerDetailInfo.accountTier);
			this.accountCategory.setValue(customerDetailInfo.accountCategory);
			this.activeCustomer.setValue(customerDetailInfo.activeCustomer);
			this.idChecked.setValue(customerDetailInfo.idChecked);
			this.street.setValue(customerDetailInfo.street);
			this.city.setValue(customerDetailInfo.city);
			this.state.setValue(customerDetailInfo.state);
			this.zip.setValue(customerDetailInfo.zip);
			this.county.setValue(customerDetailInfo.county);
			this.country.setValue(customerDetailInfo.country);
			this.addressType.setValue(customerDetailInfo.addressType);
			this.homePhone.setValue(customerDetailInfo.homePhone);
			this.businessPhone.setValue(customerDetailInfo.businessPhone);
			this.mobilePhone.setValue(customerDetailInfo.mobilePhone);
			this.email.setValue(customerDetailInfo.email);
			this.alerts.setValue(customerDetailInfo.alerts);
		});
	}

	ngAfterViewInit(): void {
		this.richTextEditor = new Jodit('#jodit-editor', {});
	}

	// canDeactivate for unsaved changes guard
	public async canDeactivate() {
		if (this.editForm.dirty) {
			const promptResult =  this.unsavedChangesService.promptUnsavedChangesDialog(this.editForm.valid);
			
			let promptUserResult = await lastValueFrom(promptResult);
			
			if (promptUserResult === 'Save') {
				// Save changes in database, then return true
				console.log('Saving changes');
				alert('The user chose to save changes, so you should capture the changes and save them to a database here.');
				return true;
			} else if (promptUserResult === 'DontSave') {
				console.log('Dont save changes');
				return true;
			} else if (promptUserResult === 'LoseChanges') {
				console.log('Losing changes');
				return true;
			} else {
				console.log('Returning to editing without leaving page');
				return false;
			};
		} else {
			return true;
		}
	}

	// Save customer changes
	public saveChanges(): void {
		if (this.editForm.valid) {
			this.postingCustomerInfo = true;
			this.dataService.postCustomerDetailInfo(this.customerInfo).then((errorFlag) => {
				this.postingCustomerInfo = false;
				if (!errorFlag) {
					alert('All fields were successfully validated, so your app would typically navigate to a different view here');
					this.editForm.markAsPristine();
					this.editForm.markAsUntouched();
				}
			});
		}
	}

	// Cancel customer changes
	public cancelChanges(): void {
		alert('This cancels the edit session, so your app would typically navigate to a different view here');
	}

	public DisplayMonthlyIncomeDialog(): void {
		this.annualIncome = this.grossIncome.value;
		this.monthlyIncome = this.annualIncome / 12;
		this.showDialog = true;
	}

	public updateAnnualIncome(): void {
		this.annualIncome = this.monthlyIncome * 12;
	}

	public closeDialog(e): void {
		if (e.detail === 'Save') {
			this.grossIncome.setValue(this.annualIncome);
		}
	}

	public addValue(event: MatChipInputEvent): void {
		const input = event.input;
		const value = event.value;

		// Add our value
		if ((value).trim()) {
			this.selectedValues.push(value.trim());
		}

		// Reset the input value
		if (input) {
			input.value = '';
		}
	}

	public removeValue(valueToRemove: string): void {
		const index = this.selectedValues.indexOf(valueToRemove);
		if (index >= 0) {
			this.selectedValues.splice(index, 1);
		}
	}

	public addTag(event: MatChipInputEvent): void {
		// If typeahead is running, skip adding string
		setTimeout(() => {
			if (!this.typeaheadSelectionInProgess) {
				this.addSelectedValueString(event.value.trim());
			}
		}, 100);

		// Reset to false for next value submission
		this.typeaheadSelectionInProgess = false;
	}

	public onTypeaheadSelect(e: any): void {
		// Stop adding tag if typeahead option is available
		this.typeaheadSelectionInProgess = true;
		this.addSelectedValueString(e.item.trim());
	}

	public addSelectedValueString(value: string): void {
		// Push value to selected values array
		if (value) {
			this.selectedValues.push(value);
		}

		// Reset input value
		this.selectedAnimal.setValue('');
	}

	public goToFormControl(formControlName: string): void {
		setTimeout(() => { this.focusField(formControlName); }, 100);
	}

	public focusField(formControlName: string): void {
		const nextControl = document.getElementById('field-' + formControlName);
		if (nextControl.getElementsByTagName('input')[0]) {
			// If element has child inputs, set focus on the first child input
			nextControl.getElementsByTagName('input')[0].focus();
		} else {
			// This is a standard input, just set focus
			nextControl.focus();
		}
	}

	public openPattern(url: string): void {
		const win = window.open(url, '_blank');
		win.focus();
	}
}
