import { Component, OnInit, OnDestroy } from '@angular/core';
import { SelectItem } from 'primeng/api';
import { UserSearchResult } from './user-search-result';
import { UserSearchService } from '../../../services/users/users.service';
import { takeWhile } from 'rxjs/operators';
import jsPDF from 'jspdf';
import autoTable from 'jspdf-autotable';
import * as xlsx from 'xlsx';
import { saveAs } from 'file-saver';

@Component({
	// tslint:disable-next-line:component-selector
	selector: 'grid',
	templateUrl: './grid.component.html',
	styleUrls: ['./grid.component.scss'],
	providers: [UserSearchService]
})
export class GridComponent implements OnInit, OnDestroy {
	public stacked: '';
	public users: UserSearchResult[];
	public selectedUsers: UserSearchResult;
	public selectedUsers1: UserSearchResult;
	public cols: any[];
	public exportColumns: any[];
	public columnOptions: SelectItem[];
	public loadError: string = '';
	private alive: boolean = true;
	public stylingOption: string = 'minimal';
	public filterCols: any[];
	public filterData: any[];
	public phoneTypes: any[];
	public userTypes: any[];
	public fileTypeList: SelectItem[];
	public groupedUsers: UserSearchResult[];
	public rowGroupMetadata: any;

	constructor(private userService: UserSearchService) { }

	ngOnInit(): void {

		this.startUserSearch();
		this.startGroupedUserSearch();

		this.cols = [
			{ field: 'Name', header: 'Name' },
			{ field: 'Phone', header: 'Phone' },
			{ field: 'Email', header: 'Email' },
			{ field: 'Address', header: 'Address' },
			{ field: 'City', header: 'City' },
			{ field: 'Region', header: 'Region' },
			{ field: 'Country', header: 'Country' },
			{ field: 'Zip', header: 'Zip' },
			{ field: 'Company', header: 'Company' }
		];

		this.exportColumns = this.cols.map(col => ({title: col.header, dataKey: col.field}));

		this.columnOptions = [];
		for (let i = 0; i < this.cols.length; i++) {
			this.columnOptions.push({ label: ' ' + this.cols[i].header, value: this.cols[i] });
		}

		this.filterCols = [
			{ field: 'input', header: 'Name (Input Filter)' },
			{ field: 'inputSearch', header: 'Phone (Search Input Filter)' },
			{ field: 'select', header: 'Phone Type (Select Filter)' },
			{ field: 'pdropdown', header: 'User Type (p-dropdown Filter)' },
			{ field: 'pmultiSelect', header: 'File Type (p-multiSelect Filter)' },
			{ field: 'menuFilter', header: 'Title (Menu Filter)' }
		];

		this.filterData = [
			{ input: 'Calista Valencia', inputSearch: '1-469-566-5631', select: 'Mobile', pdropdown: 'Customer', pmultiSelect: 'Folder', menuFilter: 'CEO' },
			{ input: 'Thor Knight', inputSearch: '1-250-948-1493', select: 'Home', pdropdown: 'Customer', pmultiSelect: 'Image', menuFilter: 'President' },
			{ input: 'Delilah Wagner', inputSearch: '1-950-215-5720', select: 'Mobile', pdropdown: 'Employee', pmultiSelect: 'Application', menuFilter: 'Marketing' },
			{ input: 'Imogene Frederick', inputSearch: '1-101-501-2521', select: 'Work', pdropdown: 'Customer', pmultiSelect: 'Video', menuFilter: 'Sales' },
			{ input: 'Fitzgerald Riddle', inputSearch: '1-860-640-5382', select: 'Home', pdropdown: 'Employee', pmultiSelect: 'PDF', menuFilter: 'Manager' },
			{ input: 'Indira Pittman', inputSearch: '1-778-680-6691', select: 'Mobile', pdropdown: 'Customer', pmultiSelect: 'Folder', menuFilter: 'Vice President' },
			{ input: 'Myra Tillman', inputSearch: '1-109-475-8137', select: 'Mobile', pdropdown: 'Customer', pmultiSelect: 'Image', menuFilter: 'Sales' },
			{ input: 'Kasimir Blevins', inputSearch: '1-315-426-7582', select: 'Home', pdropdown: 'Customer', pmultiSelect: 'Video', menuFilter: 'Sales' },
			{ input: 'Yvonne William', inputSearch: '1-988-841-1918', select: 'Work', pdropdown: 'Employee', pmultiSelect: 'Text', menuFilter: 'Vice President' }
		];

		this.phoneTypes = [
			{ label: 'All Phone Types', value: null },
			{ label: 'Mobile', value: 'Mobile' },
			{ label: 'Home', value: 'Home' },
			{ label: 'Work', value: 'Work' }
		];

		this.userTypes = [
			{ label: 'All User Types', value: null },
			{ label: 'Customer', value: 'Customer' },
			{ label: 'Employee', value: 'Employee' }
		];

		this.fileTypeList = [
			{ label: 'Application', value: 'Application' },
			{ label: 'Folder', value: 'Folder' },
			{ label: 'Image', value: 'Image' },
			{ label: 'PDF', value: 'PDF' },
			{ label: 'Text', value: 'Text' },
			{ label: 'Video', value: 'Video' }
		];
	}

	ngOnDestroy(): void {
		this.alive = false;
	}

	private startUserSearch(): any {
		this.userService.GetAllUsers().pipe(takeWhile(() => this.alive)).subscribe(
			data => {
				this.users = data;
			},
			error => this.loadError = <any>error
		);
	}

	private startGroupedUserSearch(): any {
		this.userService.GetAllUsers().pipe(takeWhile(() => this.alive)).subscribe(
			data => {
				data.sort((a, b) => {
					return a.Name < b.Name ? -1 : 1;
				});
				data.sort((a, b) => {
					return a.Company < b.Company ? -1 : 1;
				});
				this.groupedUsers = data;
			},
			error => this.loadError = <any>error
		);
		this.updateRowGroupMetaData(this.groupedUsers);
	}

	public updateRowGroupMetaData(data): any {
		this.rowGroupMetadata = {};
		if (data) {
			for (let i = 0; i < data.length; i++) {
				const rowData = data[i];
				const company = rowData.Company;
				if (i === 0) {
					this.rowGroupMetadata[company] = { index: 0, size: 1 };
				} else {
					const previousRowData = data[i - 1];
					const previousRowGroup = previousRowData.Company;
					if (company === previousRowGroup) {
						this.rowGroupMetadata[company].size++;
					} else {
						this.rowGroupMetadata[company] = { index: i, size: 1 };
					}
				}
			}

			data.sort((a, b) => {
				return a.Company < b.Company ? -1 : 1;
			});
		}
	}

	public onFilter(event: any): void {
		this.updateRowGroupMetaData(event.filteredValue);
	}

	public customSort(event: any): any {
		event.data.sort((data1, data2) => {
			const value1 = data1[event.field];
			const value2 = data2[event.field];
			let result = null;

			if (value1 == null && value2 != null) {
				result = -1;
			} else if (value1 != null && value2 == null) {
				result = 1;
			} else if (value1 == null && value2 == null) {
				result = 0;
			} else if (typeof value1 === 'string' && typeof value2 === 'string') {
				result = value1.localeCompare(value2);
			} else {
				result = (value1 < value2) ? -1 : (value1 > value2) ? 1 : 0;
			}
			return (event.order * result);
		});

		event.data.sort((a, b) => {
			return a.Company < b.Company ? -1 : 1;
		});

	}

	public exportPdf(): void {
		const doc = new jsPDF();
		autoTable(doc, { html: '#singleSelectionTable table' });
		doc.save('users.pdf');
	}

	public exportExcel(): void {
		const worksheet = xlsx.utils.json_to_sheet(this.users);
		const workbook = { Sheets: { 'data': worksheet }, SheetNames: ['data'] };
		const excelBuffer: any = xlsx.write(workbook, { bookType: 'xlsx', type: 'array' });
		this.saveAsExcelFile(excelBuffer, 'users');
	}

	public saveAsExcelFile(buffer: any, fileName: string): void {
		const EXCEL_TYPE = 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet;charset=UTF-8';
		const EXCEL_EXTENSION = '.xlsx';
		const data: Blob = new Blob([buffer], {
			type: EXCEL_TYPE
		});
		saveAs(data, fileName + '_export_' + new Date().getTime() + EXCEL_EXTENSION);
	}

	public openPattern(url: string): void {
		const win = window.open(url, '_blank');
		win.focus();
	}

}
