import { Component, OnInit } from '@angular/core';
import { DataService } from '../../../services/data/data.service';
import { Employee } from '../../../classes/employee';

@Component({
    selector: 'large-datasets',
    templateUrl: './large-datasets.component.html',
    styleUrls: ['./large-datasets.component.scss']
})
export class LargeDatasetsComponent implements OnInit {
    public approach: string = 'Pagination';
    public employeePageIndex: number = 1;
    public employeesPerPage: number = 8;
    public employeeDataset: Employee[] = null;
    public employeeTotalCount: number = 0;
    public paginationInfo: string = 'loading';
    public loading: boolean = false;

    constructor(private dataService: DataService) { }

    ngOnInit(): void {
        this.dataService.GetEmployeeTotalCount().then(
            data => {
                this.employeeTotalCount = data;
                this.employeePageIndex = 1;
                this.updatePaginationEmployeeList();
            });
    }

    public approachChanged(e): void {
        this.approach = e.detail;
        if (this.approach === 'Pagination') {
            this.setUpPagination();
        } else {
            this.setUpInfiniteScroll();
        }
    }

    public pageChanged(e) {
        this.employeePageIndex = e.detail;
        this.updatePaginationEmployeeList();
    }

    public setUpPagination(): void {
        this.employeePageIndex = 1;
        this.updatePaginationEmployeeList();
    }

    public setUpInfiniteScroll(): void {
        this.employeePageIndex = 1;
        this.updatePaginationEmployeeList();
    }

    scrolledToBottom() {
        if (this.approach === 'Infinite Scroll') {
            this.addMoreEmployees();
        }
    }

    private updatePaginationEmployeeList(): void {
        const startIndex = Math.max((this.employeePageIndex - 1) * this.employeesPerPage, 0);
        const endIndex = Math.min(startIndex + this.employeesPerPage - 1, this.employeeTotalCount - 1);
        this.paginationInfo = `Employees ${startIndex + 1} - ${endIndex + 1} of ${this.employeeTotalCount}`;

        this.loading = true;
        this.dataService.GetEmployees(startIndex, this.employeesPerPage).then(
            data => {
                this.employeeDataset = data;
                this.loading = false;
            });
    }

    private addMoreEmployees(): void {
        if (this.approach === 'Infinite Scroll' && !this.loading) {
            const startIndex = this.employeeDataset.length;
            let count = this.employeesPerPage;
            if (startIndex + count > this.employeeTotalCount) {
                count = this.employeeTotalCount - startIndex;
            }
            if (count > 0) {
                this.loading = true;
                this.dataService.GetEmployees(startIndex, count).then(
                    data => {
                        this.employeeDataset = this.employeeDataset.concat(data);
                        this.paginationInfo = 'Employees 1 - ' + this.employeeDataset.length.toString() + ' of ' + this.employeeTotalCount.toString();
                        this.loading = false;
                    }
                );
            }
        }
    }

    public openPattern(url: string): void {
        const win = window.open(url, '_blank');
        win.focus();
    }

}
