import { Component, OnInit } from '@angular/core';
import { SelectItem } from 'primeng/api';
import { UserAccess } from '../../../classes/user-access';

@Component({
	selector: 'list-categorization',
	templateUrl: './list-categorization.component.html',
	styleUrls: ['./list-categorization.component.scss']
})
export class ListCategorizationComponent implements OnInit {
	public userAccessList: UserAccess[];
	public sortedUserAccessList: UserAccess[];
	public sortColumn: string;
	public sortDirection: number = 1;
	public sortDescription: string = '';
	public sourceProducts1: any[];
	public targetProducts1: any[];
	public sourceProducts2: any[];
	public targetProducts2: any[];
	public accessLevelDropdownOptions: SelectItem[] = 	[
		{ label: 'None', value: 'None' },
		{ label: 'Viewer', value: 'Viewer' },
		{ label: 'Operator', value: 'Operator' },
		{ label: 'Admin', value: 'Admin' }
	];

	ngOnInit(): void {
		// Single list data
		this.userAccessList = [
			new UserAccess('Jeffrey Phillips', 'Operator'),
			new UserAccess('Tammy Pope', 'Viewer'),
			new UserAccess('Candice Alverado', 'Viewer')
		];
		this.sortedUserAccessList = this.userAccessList.slice();

		// Pick list data
		this.sourceProducts1 = [
			{
				'id': '1000',
				'name': 'Bamboo Watch',
				'price': 65,
				'category': 'Accessories',
				'inventoryStatus': 'In Stock'
			},
			{
				'id': '1001',
				'name': 'Black Watch',
				'price': 72,
				'category': 'Accessories',
				'inventoryStatus': 'In Stock'
			},
			{
				'id': '1002',
				'name': 'Blue Band',
				'price': 79,
				'category': 'Fitness',
				'inventoryStatus': 'Low Stock'
			},
			{
				'id': '1003',
				'name': 'Blue T-Shirt',
				'price': 29,
				'category': 'Clothing',
				'inventoryStatus': 'In Stock'
			},
			{
				'id': '1004',
				'name': 'Bracelet',
				'price': 15,
				'category': 'Accessories',
				'inventoryStatus': 'In Stock'
			},
			{
				'id': '1005',
				'name': 'Brown Purse',
				'price': 120,
				'category': 'Accessories',
				'inventoryStatus': 'Out of Stock'
			},
			{
				'id': '1006',
				'name': 'Chakra Bracelet',
				'price': 32,
				'category': 'Accessories',
				'inventoryStatus': 'Low Stock'
			},
			{
				'id': '1007',
				'name': 'Galaxy Earrings',
				'price': 34,
				'category': 'Accessories',
				'inventoryStatus': 'In Stock'
			},
			{
				'id': '1008',
				'name': 'Game Controller',
				'price': 99,
				'category': 'Electronics',
				'inventoryStatus': 'Low Stock'
			},
			{
				'id': '1009',
				'name': 'Gaming Set',
				'price': 299,
				'category': 'Electronics',
				'inventoryStatus': 'In Stock'
			}
		];

		this.targetProducts1 = [];
		this.sourceProducts2 = this.sourceProducts1.slice();
		this.targetProducts2 = [];
	}

	// Sort the list
	public sortList(sortColumn: string): any {
		// Reverse the sort direction
		this.sortDirection *= -1;
		const sortDirection: number = this.sortDirection;

		// Remember the sort column
		this.sortColumn = sortColumn;

		this.sortedUserAccessList.sort((itemA: UserAccess, itemB: UserAccess) => {
			// Compare ascendening
			let sortResult = 0;
			if (itemA[sortColumn] < itemB[sortColumn]) {
				sortResult = -1;
			} else if (itemA[sortColumn] > itemB[sortColumn]) {
				sortResult = 1;
			}

			// Adjust for sort direction
			sortResult *= sortDirection;

			return sortResult;
		});
	}

	public openPattern(url: string): void {
		const win = window.open(url, '_blank');
		win.focus();
	}

}
