import { Component, HostListener } from '@angular/core';

@Component({
	// tslint:disable-next-line:component-selector
	selector: 'notifications',
	templateUrl: './notifications.component.html',
	styleUrls: ['./notifications.component.scss']
})
export class NotificationsComponent {
	public showSuccessPopupNotification: boolean = false;
	public showInformationPopupNotification: boolean = false;
	public showWarningPopupNotification: boolean = false;
	public showErrorPopupNotification: boolean = false;
	public showStickyPopupNotification: boolean = false;
	public showButtonPopupNotification: boolean = false;

    public showSuccessBanner() {
        this.showBanner('success');
    }

    public showInformationBanner() {
        this.showBanner('information');
    }

    public showWarningBanner() {
        this.showBanner('warning');
    }

    public showErrorBanner() {
        this.showBanner('error');
    }

    private showBanner(bannerType) {
        // Construct banner detail
        const bannerDetail = {
            bannerType: bannerType,
            bannerText: `${bannerType} banner notification`
         };

        // Fire custom event: jha-app handles this event and displays the banner notification
        let showBannerEvent = new CustomEvent('rui-show-banner-notification', {
            detail: JSON.stringify(bannerDetail),
            bubbles: true,
            composed: true
        });
        dispatchEvent(showBannerEvent);
    }

    public hideBanner() {
        // Fire custom event: jha-app handles this event and hides the banner notification
        let hideBannerEvent = new CustomEvent('rui-hide-banner-notification', {
            detail: {},
            bubbles: true,
            composed: true
        });
        dispatchEvent(hideBannerEvent);
    }

    public popupNotificationClick(e) {
        const clickTree = (e.composedPath() || []).filter(item => item.localName === 'rui-popup-notification');
        if (clickTree.length > 0) {
            console.log(clickTree[0].getAttribute('id') + ' notification was clicked');
        }
    }

    public popupNotificationCloseButtonClick(e) {
        const clickTree = (e.composedPath() || []).filter(item => item.localName === 'rui-popup-notification');
        if (clickTree.length > 0) {
            console.log(clickTree[0].getAttribute('id') + ' notification\'s close button was clicked');
        }
    }

    // Handle event that specifies when the banner notification is shown or hidden
    @HostListener('window:rui-banner-notification-is-visible-change', ['$event'])
	private bannerClosed(e) {
		console.log('Banner notification is now ' + (e.detail ? 'shown' : 'hidden'));
	}

	public openPattern(url: string) {
		const win = window.open(url, '_blank');
		win.focus();
	}
}
