import { Component, HostListener, OnInit } from '@angular/core';
import { DomSanitizer } from '@angular/platform-browser';

@Component({
  selector: 'plug-in',
  templateUrl: './plug-in.component.html',
  styleUrls: ['./plug-in.component.scss']
})
export class PlugInComponent implements OnInit {
	public baseUrl: string = 'https://jhapluginhosted.z19.web.core.windows.net/';
	public displayedUrl: string = 'jhapluginhosted.z19.web.core.windows.net';
	public hostedUrl: string = '';
	public currentThemeColor: string = 'pacific';

	constructor(public sanitizer: DomSanitizer) {}

	ngOnInit(): void {
		// Get current theme color
        if (typeof (Storage) !== "undefined") {
            let themeFile = localStorage.getItem("rui-theme");
            if (themeFile) {
                this.currentThemeColor = themeFile;
            }
        }

		this.updateUrl();
	}

    @HostListener('window:rui-load-theme', ['$event'])
    private themeChanged(e) {
        this.currentThemeColor = e.detail;
        this.updateUrl();
    }

	public openPattern(url: string): void {
		const win = window.open(url, '_blank');
		win.focus();
	}

	public updateUrl(): void {
		let url = '';
		url = this.baseUrl + '?themeColor=' + this.currentThemeColor + '&host=responsive';

		this.hostedUrl = this.cleanURL(url);
	}

	private cleanURL(oldURL: string): any {
		return this.sanitizer.bypassSecurityTrustResourceUrl(oldURL);
	}

}
