import { Component, OnInit, OnDestroy } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { SearchResult } from '../../../classes/search-result';
import { SearchService } from '../../../services/search/search.service';

@Component({
	// tslint:disable-next-line:component-selector
	selector: 'search',
	templateUrl: './search.component.html',
	styleUrls: ['./search.component.scss'],
	providers: [SearchService]
})
export class SearchComponent implements OnInit, OnDestroy {
	public title: string = 'Search';
	public searching: boolean = false;
	public displaySearchInputForPhone: boolean = false;
	public noSearchResults: boolean = false;
	public blankSearchString: boolean = false;
	public searchString: string = '';
	public searchResultList: SearchResult[] = [];
	private routeSubscription: any = null;

	constructor(
		// tslint:disable-next-line:no-shadowed-variable
		private SearchService: SearchService,
		private route: ActivatedRoute,
		private router: Router
	) { }

	public ngOnInit(): void {
		this.routeSubscription = this.route.params.subscribe(params => {
			// Display a search input in the page if this is a phone-based search
			// since we don't display the search in the header at phone size.
			this.displaySearchInputForPhone = this.router.url.includes('SearchPhone');

			// Look for the search string sent via parameter and perform the search,
			// *unless* we're displaying the search input, in which case the search
			// will start when the user presses the Search button.
			if (!this.displaySearchInputForPhone) {
				this.searchString = params['search'] || '';
				this.performSearch();
			}
		});
	}

	public performSearch(): void {
		if (!this.searchString) {
			// Don't show the blank search text error for a phone search
			// since the user potentially hasn't started the search yet
			this.blankSearchString = !this.displaySearchInputForPhone;
			this.title = 'Search';
			this.searchResultList = [];
		} else {
			this.blankSearchString = false;
			this.searching = true;
			this.searchResultList = this.SearchService.getSearchResults(this.searchString.toLowerCase());
			this.searching = false;
			this.noSearchResults = this.searchResultList.length < 1;
			if (this.searchResultList.length === 1) {
				// 1 match, so just go there
				this.router.navigate([this.searchResultList[0].routerLink]);
			} else if (this.searchResultList.length < 1) {
				// No matches
				this.title = 'We didn\'t find anything that matched \'' + this.searchString + '\'';
			} else {
				// Display search results
				this.title = this.searchResultList.length + ' result' + (this.searchResultList.length === 1 ? '' : 's') +
					' found for \'' + this.searchString + '\'';
			}
		}
	}

	public ngOnDestroy(): void {
		if (this.routeSubscription) {
			this.routeSubscription.unsubscribe();
		}
	}

	public openPattern(url: string): void {
		const win = window.open(url, '_blank');
		win.focus();
	}

}
