import { Component } from '@angular/core';
import { Router } from '@angular/router';

@Component({
	selector: 'session-timeout',
	templateUrl: './session-timeout.component.html',
	styleUrls: ['./session-timeout.component.scss']
})
export class SessionTimeoutComponent {
    public sessionTimeoutRemaining = 15;
    public sessionTimeoutInterval = null;

	constructor(private router: Router) { }

    public showSessionTimeout() {
        // Reset the product-level timer that mocks up a session timeout based on user inactivity
        this.resetSessionTimeout();

        // Show the session timeout warning with a countdown time of 15 seconds, a callback
        // for the "Extend my Session" button, and a callback for the "Log Out" button
        let showSessionTimeoutEvent = new CustomEvent('rui-show-session-timeout',
            {
                detail: {
                    timeoutRemaining: this.getRemainingTimeText(),
                    extendSessionCallback: () => { this.extendSession() },
                    logOutCallback: () => { this.logOut() }
                },
                bubbles: true,
                composed: true
            });
        dispatchEvent(showSessionTimeoutEvent);

        // Set timer that mocks up a session timeout based on user inactivity
        this.sessionTimeoutInterval = setInterval(() => {
            // Count down by 1
            this.sessionTimeoutRemaining--;

            if (this.sessionTimeoutRemaining> 0) {
                // Update the session timeout warning with the remaining countdown
                let updateSessionTimeoutEvent = new CustomEvent('rui-update-session-timeout',
                    {
                        detail: {
                            timeoutRemaining: this.getRemainingTimeText(),
                        },
                        bubbles: true,
                        composed: true
                    });
                dispatchEvent(updateSessionTimeoutEvent);
            } else {
                // Hide any existing session timeout warning already displayed
                this.hideSessionTimeout();

                // Log out the user
                this.logOut()
            }
        }, 1000);
    }

    public showSessionTimeoutNoTimer() {
        // Reset the product-level timer that mocks up a session timeout based on user inactivity
        this.resetSessionTimeout();

        // Show the session timeout warning without a countdown time, no "Extend my Session" button,
        // and a callback for the "Log Out" button
        let showSessionTimeoutEvent = new CustomEvent('rui-show-session-timeout',
            {
                detail: {
                    timeoutRemaining: 'none',
                    logOutCallback: () => { this.logOut() }
                },
                bubbles: true,
                composed: true
            });
        dispatchEvent(showSessionTimeoutEvent);
    }

    public hideSessionTimeout() {
        // Reset the product-level timer that mocks up a session timeout based on user inactivity
        this.resetSessionTimeout();

        // Manually hide the session timeout warning; note that the warning hides itself when the user
        // presses either the "Extend my Session" or "Log Out" button
        let hideSessionTimeoutEvent = new CustomEvent('rui-hide-session-timeout',
            {
                detail: {},
                bubbles: true,
                composed: true
            });
        dispatchEvent(hideSessionTimeoutEvent);
    }

    public extendSession() {
        // Reset the product-level timer that mocks up a session timeout based on user inactivity
        this.resetSessionTimeout();

        console.log('Session extended');
    }

    public logOut() {
        // Reset the product-level timer that mocks up a session timeout based on user inactivity
        this.resetSessionTimeout();

        // Code to log out the user goes here...

        // Navigate to the Log In view
        this.router.navigate(['InactivityTimeout']);
    }

    public resetSessionTimeout() {
        // Reset the dummy timeout remaining value (you wouldn't do this in production)
        this.sessionTimeoutRemaining = 15;

        // Reset the product-level timer that mocks up a session timeout based on user inactivity
        clearInterval(this.sessionTimeoutInterval);
    }

    public getRemainingTimeText() {
        return this.sessionTimeoutRemaining + ' second' + (this.sessionTimeoutRemaining != 1 ? 's' : '');
    }

	public openPattern(url: string): void {
		const win = window.open(url, '_blank');
		win.focus();
	}

}
