import { Component } from '@angular/core';
import { ruiThemeList } from '@jha/rui-wc/js/rui-theming';

@Component({
	// tslint:disable-next-line:component-selector
	selector: 'theming',
	templateUrl: './theming.component.html',
	styleUrls: ['./theming.component.scss']
})
export class ThemingComponent {
	public themeList: any[] = ruiThemeList;
	public cssVars: string[];

	constructor() {
		this.defineCssVars();
	}

	public changeTheme(loadName: string): void {
        // Fire custom event with theme name
        let themeChangeEvent = new CustomEvent('rui-load-theme', { detail: loadName, bubbles: true, composed: true });
        document.dispatchEvent(themeChangeEvent);
	}

	public openPattern(url): void {
		const win = window.open(url, '_blank');
		win.focus();
	}

	public defineCssVars(): void {

		this.cssVars =
		[
			'--accent',
			'--body-bg',
			'--box-shadow',
			'--breadcrumb',
			'--breadcrumb-display-block',
			'--breadcrumb-separator',
			'--breadcrumb-separator-display-block',
			'--btn-badge-bg-destructive',
			'--btn-badge-bg-primary',
			'--btn-badge-bg-secondary',
			'--btn-badge-fg-destructive',
			'--btn-badge-fg-primary',
			'--btn-badge-fg-secondary',
			'--btn-destructive-bg',
			'--btn-destructive-bg-active',
			'--btn-destructive-bg-disabled',
			'--btn-destructive-bg-hover',
			'--btn-destructive-border',
			'--btn-destructive-border-disabled',
			'--btn-destructive-fg',
			'--btn-destructive-fg-disabled',
			'--btn-destructive-fg-hover',
			'--btn-misc-bg',
			'--btn-misc-bg-hover',
			'--btn-misc-fg',
			'--btn-misc-fg-hover',
			'--btn-primary-bg',
			'--btn-primary-bg-active',
			'--btn-primary-bg-disabled',
			'--btn-primary-bg-hover',
			'--btn-primary-border',
			'--btn-primary-border-disabled',
			'--btn-primary-fg',
			'--btn-primary-fg-disabled',
			'--btn-primary-fg-hover',
			'--btn-secondary-bg',
			'--btn-secondary-bg-active',
			'--btn-secondary-bg-disabled',
			'--btn-secondary-bg-hover',
			'--btn-secondary-border',
			'--btn-secondary-border-disabled',
			'--btn-secondary-fg',
			'--btn-secondary-fg-disabled',
			'--btn-secondary-fg-hover',
			'--busy-indicator',
			'--busy-indicator-backdrop',
			'--chart-data-series-color-1',
			'--chart-data-series-color-2',
			'--chart-data-series-color-3',
			'--chart-data-series-color-4',
			'--chart-data-series-color-5',
			'--chart-data-series-color-6',
			'--chart-data-series-color-7',
			'--chart-data-series-color-8',
			'--chart-data-series-color-9',
			'--chart-data-series-color-10',
			'--chart-data-series-translucent-color-1',
			'--chart-data-series-translucent-color-2',
			'--chart-data-series-translucent-color-3',
			'--chart-data-series-translucent-color-4',
			'--chart-data-series-translucent-color-5',
			'--chart-data-series-translucent-color-6',
			'--chart-data-series-translucent-color-7',
			'--chart-data-series-translucent-color-8',
			'--chart-data-series-translucent-color-9',
			'--chart-data-series-translucent-color-10',
			'--chart-grid-color',
			'--content-primary-bg',
			'--content-secondary-bg',
			'--display-block-border-color',
			'--form-disabled-bg',
			'--form-disabled-fg',
			'--form-required-field',
			'--function-title',
			'--grid-btn-fg-disabled',
			'--line',
			'--line-heavy',
			'--list-item-hovered-bg',
			'--list-item-hovered-fg',
			'--list-item-selected-bg',
			'--list-item-selected-fg',
			'--main-header-bg',
			'--main-header-bg-hover',
			'--main-header-border-bottom',
			'--main-header-fg',
			'--main-nav-bg',
			'--main-nav-bg-active',
			'--main-nav-border',
			'--main-nav-fg-active',
			'--main-nav-fg-inactive',
			'--main-nav-separator',
			'--main-nav-width-collapsed',
			'--main-nav-width-expanded',
			'--modal-backdrop',
			'--modal-border',
			'--notification-error-accent',
			'--notification-error-bg',
			'--notification-error-border',
			'--notification-error-close-btn',
			'--notification-error-fg',
			'--notification-info-accent',
			'--notification-info-bg',
			'--notification-info-border',
			'--notification-info-close-btn',
			'--notification-info-fg',
			'--notification-success-accent',
			'--notification-success-bg',
			'--notification-success-border',
			'--notification-success-close-btn',
			'--notification-success-fg',
			'--notification-warning-accent',
			'--notification-warning-bg',
			'--notification-warning-border',
			'--notification-warning-close-btn',
			'--notification-warning-fg',
			'--outline-offset-anchor',
			'--outline-offset-btn',
			'--outline-style',
			'--outline-width',
			'--password-strength-meter-medium-bg',
			'--password-strength-meter-strong-bg',
			'--password-strength-meter-text-shadow',
			'--password-strength-meter-weak-bg',
			'--percentage-circle-data-active',
			'--percentage-circle-data-bg',
			'--percentage-circle-data-inactive',
			'--popup-bg',
			'--popup-border',
			'--popup-border-radius',
			'--popup-fg',
			'--popup-header-separator',
			'--popup-menu-option-padding',
			'--popup-menu-separator',
			'--scrollbar-thumb',
			'--scrollbar-track',
			'--search-box-bg',
			'--search-box-border',
			'--search-box-button-bg',
			'--search-box-button-bg-hover',
			'--search-box-button-fg',
			'--search-box-button-fg-hover',
			'--search-box-fg',
			'--search-box-watermark',
			'--slide-up-bg',
			'--slider-thumb',
			'--slider-thumb-focused',
			'--slider-track',
			'--splitter-bg',
			'--table-footer-bg',
			'--table-header-bg',
			'--table-stripe-bg',
			'--table-stripe-fg',
			'--text-bright',
			'--text-disabled',
			'--text-error',
			'--text-help',
			'--text-highlight',
			'--text-regular',
			'--tile-alt1-bg',
			'--tile-alt1-bg-hover',
			'--tile-alt1-fg',
			'--tile-alt2-bg',
			'--tile-alt2-bg-hover',
			'--tile-alt2-fg',
			'--tile-alt3-bg',
			'--tile-alt3-bg-hover',
			'--tile-alt3-fg',
			'--tile-highlighted-bg',
			'--tile-highlighted-bg-hover',
			'--tile-highlighted-fg',
			'--toast-bg',
			'--toast-close-button-border',
			'--toast-close-button-fg',
			'--toast-fg',
			'--toggle-thumb-bg',
			'--toggle-track-border',
			'--toolbar-floating-bg',
			'--toolbar-function-bg',
			'--toolbar-separator-bg-function',
			'--well-bg',
			'--well-border'
		]
	}

}
