import { Component, OnInit } from '@angular/core';
import { TreeNode } from 'primeng/api';
import { SelectItem } from 'primeng/api';

@Component({
	selector: 'tree-view',
	templateUrl: './tree-view.component.html',
	styleUrls: ['./tree-view.component.scss']
})
export class TreeViewComponent implements OnInit {
	public treeNormal: TreeNode[] = null;
	public treeCheckboxes: TreeNode[] = null;
	public selectedCheckboxes: TreeNode[] = null;
	public treeViewSection: string = 'General';
	public selectedTreeNormalStructure: TreeNode = null;
	public treeViewSections: SelectItem[] = [];

	ngOnInit(): void {

		this.treeViewSections = [
			{ label: 'General', value: 'General' },
			{ label: 'Accessibility', value: 'Accessibility' },
			{ label: 'Restrictions', value: 'Restrictions' },
			{ label: 'Language and Region', value: 'Language and Region' },
			{ label: 'Notifications', value: 'Notifications' },
			{ label: 'Internal Notifications', value: 'Internal Notifications' },
			{ label: 'External Notifications', value: 'External Notifications' },
			{ label: 'Control Center', value: 'Control Center' },
			{ label: 'Privacy', value: 'Privacy' }
		];

		// Tree data for PrimeNG Treeview
		this.treeNormal = [
			{
				'label': 'General',
				'children': [{
					'label': 'Accessibility'
				},
				{
					'label': 'Restrictions'
				},
				{
					'label': 'Language and Region'
				}
				]
			},
			{
				'label': 'Notifications',
				'children': [
					{ 'label': 'Internal Notifications' },
					{ 'label': 'External Notifications' }
				]
			},
			{
				'label': 'Control Center'
			},
			{
				'label': 'Privacy'
			}
		];

		this.treeCheckboxes = this.treeNormal;

		// Preselect some checkbox nodes
		this.selectedCheckboxes = [
			this.treeCheckboxes[1],
			this.treeCheckboxes[1].children[0],
			this.treeCheckboxes[1].children[1],
			this.treeCheckboxes[3]
		];

		this.treeNormal.forEach(node => {
			this.expandRecursive(node, true);
		});

		this.treeCheckboxes.forEach(node => {
			this.expandRecursive(node, true);
		});
	}

	// Show content based on tree selection
	public onTreeNodeSelect(e: any): void {
		this.treeViewSection = e.node.label;
	}

	// Expand Treeview
	private expandRecursive(node: TreeNode, isExpand: boolean): void {
		node.expanded = isExpand;
		if (node.children) {
			node.children.forEach(childNode => {
				this.expandRecursive(childNode, isExpand);
			});
		}
	}

	public openPattern(url: string): void {
		const win = window.open(url, '_blank');
		win.focus();
	}

}
