﻿import { Injectable } from '@angular/core';
import { SearchResult } from '../../classes/search-result';
import { ViewSearchData } from '../../classes/view-search-data';

@Injectable()
export class SearchService {
	// Search terms must be LOWERCASE!
	//
	// Also, a view will match if the user's search text is a substring of a search term,
	// so no need to supply search terms that are substrings of each other. For example, the
	// "jha-breadcrumb" search term will match the following user search text examples:
	// "breadcrumb", "bread", and "crumb" since they  are all substrings of "jha-breadcrumb".
	// It will not match the user search term "breadcrumbs" though, so we include that as well.
	private viewSearchDataList: ViewSearchData[] = [
		new ViewSearchData('Animation', '/Animation', ['animation', 'animate']),
		new ViewSearchData('Breadcrumbs', '/Breadcrumbs', ['jha-breadcrumb', 'breadcrumbs', 'layout', 'dropdown']),
		new ViewSearchData('Busy Indicator', '/BusyIndicator', ['busy indicator', 'spinner']),
		new ViewSearchData('Buttons', '/Buttons', ['buttons', 'split button', 'dropdown button', 'toggle']),
		new ViewSearchData('Carousel', '/Carousel', ['carousel', 'images']),
		new ViewSearchData('Change Password', '/ChangePassword', ['change password', 'view']),
		new ViewSearchData('Charts', '/Charts', ['charts', 'graphs', 'data']),
		new ViewSearchData('Context Menu', '/ContextMenu', ['context menu', 'right click']),
		new ViewSearchData('Content Layout', '/ContentLayout', ['content layout', 'function view', 'display blocks', 'group boxes', 'tabs', 'subheader', 'well']),
		new ViewSearchData('CRUD', '/Crud', ['crud', 'create', 'add', 'read', 'inquiry', 'edit', 'update', 'revise', 'delete', 'fields', 'view']),
		new ViewSearchData('Dashboard', '/Dashboard', ['dashboard', 'graph', 'chart', 'data', 'kpi', 'trend', 'jha-dashboard-info']),
		new ViewSearchData('Data Dense', '/DataDense', ['data dense', 'layout']),
		new ViewSearchData('Dialog Box', '/DialogBox', ['dialog box', 'jha-dialog-box', 'jha-question', 'jha-dialog-box-button', 'modal']),
		new ViewSearchData('Display Block', '/DisplayBlock', ['display block', 'layout', 'jha-display-block']),
		new ViewSearchData('Drag and Drop', '/DragAndDrop', ['drag and drop']),
		new ViewSearchData('Editing Form Data', '/EditingFormData', ['editing form data', 'input', 'validation', 'validate', 'validating', 'fields', 'toggle', 'date', 'time', 'picker', 'datepicker', 'typeahead', 'rich', 'currency', 'masked', 'rate', 'list', 'select', 'multiselect', 'regex', 'action button', 'tooltip', 'more', 'information']),
		new ViewSearchData('Enterprise Icons', '/EnterpriseIcons', ['enterprise icons', 'xj-icon']),
		new ViewSearchData('File Upload', '/FileUpload', ['file upload', 'files', 'upload', 'import', 'form']),
		new ViewSearchData('Fixed Layout', '/FixedLayout', ['fixed layout']),
		new ViewSearchData('Fixed Layout Splitter', '/FixedLayoutSplitter', ['fixed layout', 'splitter']),
		new ViewSearchData('Data Grid', '/Grid', ['grid', 'turbo', 'table', 'data']),
		new ViewSearchData('Data Grid: Tree', '/GridTree', ['grid', 'tree', 'table', 'data']),
		new ViewSearchData('Data Grid: Complex', '/GridComplex', ['grid', 'complex', 'ag-grid', 'data']),
		new ViewSearchData('Group Box', '/GroupBox', ['group box', 'layout']),
		new ViewSearchData('Headshot Photos', '/HeadshotPhotos', ['headshot', 'photos', 'circle']),
		new ViewSearchData('Large Datasets', '/LargeDatasets', ['large datasets', 'infinite scrolling', 'pagination']),
		new ViewSearchData('Large Forms: Wizard', '/LargeFormsWizard', ['large forms', 'wizard', 'input', 'validation', 'validate', 'validating', 'fields']),
		new ViewSearchData('Large Forms: Accordion', '/LargeFormsAccordion', ['large forms', 'wizard', 'input', 'validation', 'validate', 'validating', 'fields']),
		new ViewSearchData('Lazy Load', '/LazyLoad', ['lazy load', 'modules']),
		new ViewSearchData('Log In', '/LogIn', ['log in', 'login', 'log out', 'logout', 'view']),
		new ViewSearchData('Master/Detail: Buttons', '/MasterDetailButtons', ['master detail', 'master/detail', 'master / detail', 'view', 'button']),
		new ViewSearchData('Master/Detail: Data Grid', '/MasterDetailGrid', ['master detail', 'master/detail', 'master / detail', 'view', 'grid']),
		new ViewSearchData('Master/Detail: Data Grid Multiple Selection', '/MasterDetailGridMultiple', ['master detail', 'master/detail', 'master / detail', 'view', 'grid', 'multiple', 'selection']),
		new ViewSearchData('Master/Detail: Tree View', '/MasterDetailTreeView', ['master detail', 'master/detail', 'master / detail', 'view', 'tree']),
		new ViewSearchData('Notifications', '/Notifications', ['notifications', 'inline notification', 'popup notification', 'banner notification', 'jha-inline-notification', 'jha-notification', 'jha-popup-notification', 'success', 'information', 'warning', 'error']),
		new ViewSearchData('Percentage Circle', '/PercentageCircle', ['percentage circle', 'jha-percentage-circle', 'data']),
		new ViewSearchData('List Categorization', '/ListCategorization', ['list', 'categorization', 'category', 'button', 'group', 'select', 'pick', 'sort', 'sortable', 'data', 'drag and drop']),
		new ViewSearchData('Process Timeline', '/ProcessTimeline', ['process-timeline', 'step', 'steps', 'jha-process-timeline', 'jha-process-timeline-step', 'jha-process-timeline-substep']),
		new ViewSearchData('Progress Bar', '/ProgressBar', ['progress bar', 'jha-progress-bar']),
		new ViewSearchData('Plug In Apps', '/PlugIn', ['plug-in', 'plug in', 'application', 'host', 'hosted', 'iframe', 'embedded', 'browser']),
		new ViewSearchData('Rating', '/Rating', ['ratings', 'star', 'jha-rating', 'data']),
		new ViewSearchData('Record Detail', '/RecordDetail', ['record detail', 'data', 'fields', 'field labels', 'field values', 'subheader']),
		new ViewSearchData('Responsive Layout', '/ResponsiveLayout', ['responsive design', 'layout']),
		new ViewSearchData('Responsive Table', '/ResponsiveTable', ['grid', 'table', 'data', 'responsive']),
		new ViewSearchData('Session Timeout', '/SessionTimeout', ['session timeout', 'inactivity', 'JhaSessionTimeoutService']),
		new ViewSearchData('Slider', '/Slider', ['slider', 'input', 'range', 'data']),
		new ViewSearchData('Status Icon', '/StatusIcon', ['status icon', 'state']),
		new ViewSearchData('Steps', '/Steps', ['steps', 'view']),
		new ViewSearchData('Tabs', '/Tabs', ['tabs', 'mattabs', 'mat-tab-group', 'mat-tab', 'layout']),
		new ViewSearchData('Tags', '/Tags', ['jha-tags']),
		new ViewSearchData('Templated Lists', '/TemplatedLists', ['templated lists', 'data']),
		new ViewSearchData('Text', '/Text', ['text', 'regular', 'bright', 'error', 'data']),
		new ViewSearchData('Themed SVG Images', '/ThemedSvgImages', ['themed svg images']),
		new ViewSearchData('Theming', '/Theming', ['theming', 'theme', 'colors']),
		new ViewSearchData('Tiles', '/Tiles', ['tiles', 'buttons', 'jha-tile']),
		new ViewSearchData('Toolbar', '/Toolbar', ['toolbars', 'function toolbar', 'floating toolbar', 'separator', 'jha-floating-toolbar', 'jha-function-toolbar-right', 'jha-display-block-toolbar-right', 'jha-toolbar-separator', 'layout']),
		new ViewSearchData('Tooltips', '/Tooltips', ['tooltips', 'more', 'information']),
		new ViewSearchData('Tree View', '/TreeView', ['treeview', 'tree view', 'data']),
		new ViewSearchData('Unsaved Changes', '/UnsavedChanges', ['unsaved change', 'promptUnsavedChangesDialog', 'DontSave', 'ContinueEditing']),
		new ViewSearchData('Wizard', '/Wizard', ['wizards', 'steps', 'view'])
	];

	public getSearchResults(searchString: string): SearchResult[] {
		const resultList: SearchResult[] = [];

		this.viewSearchDataList.forEach((searchData) => {
			let found: boolean = false;
			for (let i = 0; i < searchData.searchTerms.length && !found; i++) {
				if (searchData.searchTerms[i].includes(searchString)) {
					found = true;
					resultList.push(new SearchResult(searchData.title, searchData.routerLink));
				}
			}
		 });

		return resultList;
	}
}
